# ***************************************************************************
# *   (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>                  *
# *   (c) 2009, 2010 Ken Cline <cline@frii.com>                             *
# *   (c) 2020 Eliud Cabrera Castillo <e.cabrera-castillo@tum.de>           *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful,            *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with FreeCAD; if not, write to the Free Software        *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides tools for creating dimension objects with the Draft Workbench.

The objects can be simple linear dimensions that measure between two arbitrary
points, or linear dimensions linked to an edge.
It can also be radius or diameter dimensions that measure circles
and circular arcs.
And it can also be an angular dimension measuring the angle between
two straight lines.
"""
## @package gui_texts
# \ingroup DRAFT
# \brief Provides tools for creating dimensions with the Draft Workbench.

import math
from PySide.QtCore import QT_TRANSLATE_NOOP
import sys

import FreeCAD as App
import FreeCADGui as Gui
import Draft_rc
import DraftVecUtils
import draftguitools.gui_base_original as gui_base_original
import draftguitools.gui_tool_utils as gui_tool_utils
import draftguitools.gui_trackers as trackers
from draftutils.translate import translate
from draftutils.messages import _msg

# The module is used to prevent complaints from code checkers (flake8)
True if Draft_rc.__name__ else False


class Dimension(gui_base_original.Creator):
    """Gui command for the Dimension tool.

    This includes at the moment linear, radial, diametrical,
    and angular dimensions depending on the selected object
    and the modifier key (ALT) used.

    Maybe in the future each type can be in its own class,
    and they can inherit their basic properties from a parent class.
    """

    def __init__(self):
        self.max = 2
        self.cont = None
        self.dir = None

    def GetResources(self):
        """Set icon, menu and tooltip."""
        _tip = ("Creates a dimension. "
                "Select a straight line to create a linear dimension.\n"
                "Select an arc or circle to create a radius or diameter "
                "dimension.\n"
                "Select two straight lines to create an angular dimension "
                "between them.\n"
                "CTRL to snap, SHIFT to constrain, "
                "ALT to select an edge or arc.")

        return {'Pixmap': 'Draft_Dimension',
                'Accel': "D, I",
                'MenuText': QT_TRANSLATE_NOOP("Draft_Dimension", "Dimension"),
                'ToolTip': QT_TRANSLATE_NOOP("Draft_Dimension", _tip)}

    def Activated(self):
        """Execute when the command is called."""
        name = translate("draft", "Dimension")
        if self.cont:
            self.finish()
        elif self.hasMeasures():
            super(Dimension, self).Activated(name)
            self.dimtrack = trackers.dimTracker()
            self.arctrack = trackers.arcTracker()
            self.createOnMeasures()
            self.finish()
        else:
            super(Dimension, self).Activated(name)
            if self.ui:
                self.ui.pointUi(name)
                self.ui.continueCmd.show()
                self.ui.selectButton.show()
                self.altdown = False
                self.call = self.view.addEventCallback("SoEvent", self.action)
                self.dimtrack = trackers.dimTracker()
                self.arctrack = trackers.arcTracker()
                self.link = None
                self.edges = []
                self.pts = []
                self.angledata = None
                self.indices = []
                self.center = None
                self.arcmode = False
                self.point2 = None
                self.force = None
                self.info = None
                self.selectmode = False
                self.setFromSelection()
                _msg(translate("draft", "Pick first point"))
                Gui.draftToolBar.show()

    def setFromSelection(self):
        """Fill the nodes according to the selected geometry."""
        import DraftGeomUtils

        sel = Gui.Selection.getSelectionEx()
        if len(sel) == 1:
            if len(sel[0].SubElementNames) == 1:
                if "Edge" in sel[0].SubElementNames[0]:
                    edge = sel[0].SubObjects[0]
                    n = int(sel[0].SubElementNames[0].lstrip("Edge")) - 1
                    self.indices.append(n)
                    if DraftGeomUtils.geomType(edge) == "Line":
                        self.node.extend([edge.Vertexes[0].Point,
                                          edge.Vertexes[1].Point])
                        v1 = None
                        v2 = None
                        for i, v in enumerate(sel[0].Object.Shape.Vertexes):
                            if v.Point == edge.Vertexes[0].Point:
                                v1 = i
                            if v.Point == edge.Vertexes[1].Point:
                                v2 = i
                        if (v1 is not None) and (v2 is not None):
                            self.link = [sel[0].Object, v1, v2]
                    elif DraftGeomUtils.geomType(edge) == "Circle":
                        self.node.extend([edge.Curve.Center,
                                          edge.Vertexes[0].Point])
                        self.edges = [edge]
                        self.arcmode = "diameter"
                        self.link = [sel[0].Object, n]

    def hasMeasures(self):
        """Check if measurement objects are selected."""
        sel = Gui.Selection.getSelection()
        if not sel:
            return False
        for o in sel:
            if not o.isDerivedFrom("App::MeasureDistance"):
                return False
        return True

    def finish(self, closed=False):
        """Terminate the operation."""
        self.cont = None
        self.dir = None
        super(Dimension, self).finish()
        if self.ui:
            self.dimtrack.finalize()
            self.arctrack.finalize()

    def createOnMeasures(self):
        """Create on measurement objects."""
        for o in Gui.Selection.getSelection():
            p1 = o.P1
            p2 = o.P2
            _root = o.ViewObject.RootNode
            _ch = _root.getChildren()[1].getChildren()[0].getChildren()[0]
            pt = _ch.getChildren()[3]
            p3 = App.Vector(pt.point.getValues()[2].getValue())
            Gui.addModule("Draft")
            _cmd = 'Draft.makeDimension'
            _cmd += '('
            _cmd += DraftVecUtils.toString(p1) + ', '
            _cmd += DraftVecUtils.toString(p2) + ', '
            _cmd += DraftVecUtils.toString(p3)
            _cmd += ')'
            _rem = 'FreeCAD.ActiveDocument.removeObject("' + o.Name + '")'
            _cmd_list = ['dim = ' + _cmd,
                         _rem,
                         'Draft.autogroup(dim)',
                         'FreeCAD.ActiveDocument.recompute()']
            self.commit(translate("draft", "Create Dimension"),
                        _cmd_list)

    def createObject(self):
        """Create the actual object in the current document."""
        import DraftGeomUtils
        Gui.addModule("Draft")

        if self.angledata:
            normal = "None"
            if len(self.edges) == 2:
                v1 = DraftGeomUtils.vec(self.edges[0])
                v2 = DraftGeomUtils.vec(self.edges[1])
                normal = DraftVecUtils.toString((v1.cross(v2)).normalize())

            _cmd = 'Draft.makeAngularDimension('
            _cmd += 'center=' + DraftVecUtils.toString(self.center) + ', '
            _cmd += 'angles='
            _cmd += '['
            _cmd += str(self.angledata[0]) + ', '
            _cmd += str(self.angledata[1])
            _cmd += '], '
            _cmd += 'p3=' + DraftVecUtils.toString(self.node[-1]) + ', '
            _cmd += 'normal=' + normal
            _cmd += ')'
            _cmd_list = ['dim = ' + _cmd,
                         'Draft.autogroup(dim)',
                         'FreeCAD.ActiveDocument.recompute()']
            self.commit(translate("draft", "Create Dimension"),
                        _cmd_list)
        elif self.link and not self.arcmode:
            ops = []
            # Linear dimension, linked
            if self.force == 1:
                _cmd = 'Draft.makeDimension'
                _cmd += '('
                _cmd += 'FreeCAD.ActiveDocument.' + self.link[0].Name + ', '
                _cmd += str(self.link[1]) + ', '
                _cmd += str(self.link[2]) + ', '
                _cmd += DraftVecUtils.toString(self.node[2])
                _cmd += ')'
                _cmd_list = ['dim = ' + _cmd,
                             'dim.Direction = FreeCAD.Vector(0, 1, 0)',
                             'Draft.autogroup(dim)',
                             'FreeCAD.ActiveDocument.recompute()']
                self.commit(translate("draft", "Create Dimension"),
                            _cmd_list)
            elif self.force == 2:
                _cmd = 'Draft.makeDimension'
                _cmd += '('
                _cmd += 'FreeCAD.ActiveDocument.' + self.link[0].Name + ', '
                _cmd += str(self.link[1]) + ', '
                _cmd += str(self.link[2]) + ', '
                _cmd += DraftVecUtils.toString(self.node[2])
                _cmd += ')'
                _cmd_list = ['dim = ' + _cmd,
                             'dim.Direction = FreeCAD.Vector(1, 0, 0)',
                             'Draft.autogroup(dim)',
                             'FreeCAD.ActiveDocument.recompute()']
                self.commit(translate("draft", "Create Dimension"),
                            _cmd_list)
            else:
                _cmd = 'Draft.makeDimension'
                _cmd += '('
                _cmd += 'FreeCAD.ActiveDocument.' + self.link[0].Name + ', '
                _cmd += str(self.link[1]) + ', '
                _cmd += str(self.link[2]) + ', '
                _cmd += DraftVecUtils.toString(self.node[2])
                _cmd += ')'
                _cmd_list = ['dim = ' + _cmd,
                             'Draft.autogroup(dim)',
                             'FreeCAD.ActiveDocument.recompute()']
                self.commit(translate("draft", "Create Dimension"),
                            _cmd_list)
        elif self.arcmode:
            # Radius or dimeter dimension, linked
            _cmd = 'Draft.makeDimension'
            _cmd += '('
            _cmd += 'FreeCAD.ActiveDocument.' + self.link[0].Name + ', '
            _cmd += str(self.link[1]) + ', '
            _cmd += '"' + str(self.arcmode) + '", '
            _cmd += DraftVecUtils.toString(self.node[2])
            _cmd += ')'
            _cmd_list = ['dim = ' + _cmd,
                         'Draft.autogroup(dim)',
                         'FreeCAD.ActiveDocument.recompute()']
            self.commit(translate("draft", "Create Dimension"),
                        _cmd_list)
        else:
            # Linear dimension, non-linked
            _cmd = 'Draft.makeDimension'
            _cmd += '('
            _cmd += DraftVecUtils.toString(self.node[0]) + ', '
            _cmd += DraftVecUtils.toString(self.node[1]) + ', '
            _cmd += DraftVecUtils.toString(self.node[2])
            _cmd += ')'
            _cmd_list = ['dim = ' + _cmd,
                         'Draft.autogroup(dim)',
                         'FreeCAD.ActiveDocument.recompute()']
            self.commit(translate("draft", "Create Dimension"),
                        _cmd_list)
        if self.ui.continueMode:
            self.cont = self.node[2]
            if not self.dir:
                if self.link:
                    v1 = self.link[0].Shape.Vertexes[self.link[1]].Point
                    v2 = self.link[0].Shape.Vertexes[self.link[2]].Point
                    self.dir = v2.sub(v1)
                else:
                    self.dir = self.node[1].sub(self.node[0])
            self.node = [self.node[1]]
        self.link = None

    def selectEdge(self):
        """Toggle the select mode to the opposite state."""
        self.selectmode = not(self.selectmode)

    def action(self, arg):
        """Handle the 3D scene events.

        This is installed as an EventCallback in the Inventor view.

        Parameters
        ----------
        arg: dict
            Dictionary with strings that indicates the type of event received
            from the 3D view.
        """
        if arg["Type"] == "SoKeyboardEvent":
            if arg["Key"] == "ESCAPE":
                self.finish()
        elif arg["Type"] == "SoLocation2Event":  # mouse movement detection
            import DraftGeomUtils
            shift = gui_tool_utils.hasMod(arg, gui_tool_utils.MODCONSTRAIN)
            if self.arcmode or self.point2:
                gui_tool_utils.setMod(arg, gui_tool_utils.MODCONSTRAIN, False)
            (self.point,
             ctrlPoint, self.info) = gui_tool_utils.getPoint(self, arg,
                                                             noTracker=(len(self.node)>0))
            if (gui_tool_utils.hasMod(arg, gui_tool_utils.MODALT)
                    or self.selectmode) and (len(self.node) < 3):
                self.dimtrack.off()
                if not self.altdown:
                    self.altdown = True
                    self.ui.switchUi(True)
                    if hasattr(Gui, "Snapper"):
                        Gui.Snapper.setSelectMode(True)
                snapped = self.view.getObjectInfo((arg["Position"][0],
                                                   arg["Position"][1]))
                if snapped:
                    ob = self.doc.getObject(snapped['Object'])
                    if "Edge" in snapped['Component']:
                        num = int(snapped['Component'].lstrip('Edge')) - 1
                        ed = ob.Shape.Edges[num]
                        v1 = ed.Vertexes[0].Point
                        v2 = ed.Vertexes[-1].Point
                        self.dimtrack.update([v1, v2, self.cont])
            else:
                if self.node and (len(self.edges) < 2):
                    self.dimtrack.on()
                if len(self.edges) == 2:
                    # angular dimension
                    self.dimtrack.off()
                    r = self.point.sub(self.center)
                    self.arctrack.setRadius(r.Length)
                    a = self.arctrack.getAngle(self.point)
                    pair = DraftGeomUtils.getBoundaryAngles(a, self.pts)
                    if not (pair[0] < a < pair[1]):
                        self.angledata = [4 * math.pi - pair[0],
                                          2 * math.pi - pair[1]]
                    else:
                        self.angledata = [2 * math.pi - pair[0],
                                          2 * math.pi - pair[1]]
                    self.arctrack.setStartAngle(self.angledata[0])
                    self.arctrack.setEndAngle(self.angledata[1])
                if self.altdown:
                    self.altdown = False
                    self.ui.switchUi(False)
                    if hasattr(Gui, "Snapper"):
                        Gui.Snapper.setSelectMode(False)
                if self.dir:
                    _p = DraftVecUtils.project(self.point.sub(self.node[0]),
                                               self.dir)
                    self.point = self.node[0].add(_p)
                if len(self.node) == 2:
                    if self.arcmode and self.edges:
                        cen = self.edges[0].Curve.Center
                        rad = self.edges[0].Curve.Radius
                        baseray = self.point.sub(cen)
                        v2 = DraftVecUtils.scaleTo(baseray, rad)
                        v1 = v2.negative()
                        if shift:
                            self.node = [cen, cen.add(v2)]
                            self.arcmode = "radius"
                        else:
                            self.node = [cen.add(v1), cen.add(v2)]
                            self.arcmode = "diameter"
                        self.dimtrack.update(self.node)
                # Draw constraint tracker line.
                if shift and (not self.arcmode):
                    if len(self.node) == 2:
                        if not self.point2:
                            self.point2 = self.node[1]
                        else:
                            self.node[1] = self.point2
                        if not self.force:
                            _p = self.point.sub(self.node[0])
                            a = abs(_p.getAngle(App.DraftWorkingPlane.u))
                            if (a > math.pi/4) and (a <= 0.75*math.pi):
                                self.force = 1
                            else:
                                self.force = 2
                        if self.force == 1:
                            self.node[1] = App.Vector(self.node[0].x,
                                                      self.node[1].y,
                                                      self.node[0].z)
                        elif self.force == 2:
                            self.node[1] = App.Vector(self.node[1].x,
                                                      self.node[0].y,
                                                      self.node[0].z)
                else:
                    self.force = None
                    if self.point2 and (len(self.node) > 1):
                        self.node[1] = self.point2
                        self.point2 = None
                # update the dimline
                if self.node and not self.arcmode:
                    self.dimtrack.update(self.node
                                         + [self.point] + [self.cont])
            gui_tool_utils.redraw3DView()
        elif arg["Type"] == "SoMouseButtonEvent":
            if (arg["State"] == "DOWN") and (arg["Button"] == "BUTTON1"):
                import DraftGeomUtils
                if self.point:
                    self.ui.redraw()
                    if (not self.node) and (not self.support):
                        gui_tool_utils.getSupport(arg)
                    if (gui_tool_utils.hasMod(arg, gui_tool_utils.MODALT)
                            or self.selectmode) and (len(self.node) < 3):
                        # print("snapped: ",self.info)
                        if self.info:
                            ob = self.doc.getObject(self.info['Object'])
                            if 'Edge' in self.info['Component']:
                                num = int(self.info['Component'].lstrip('Edge')) - 1
                                ed = ob.Shape.Edges[num]
                                v1 = ed.Vertexes[0].Point
                                v2 = ed.Vertexes[-1].Point
                                i1 = i2 = None
                                for i in range(len(ob.Shape.Vertexes)):
                                    if v1 == ob.Shape.Vertexes[i].Point:
                                        i1 = i
                                    if v2 == ob.Shape.Vertexes[i].Point:
                                        i2 = i
                                if (i1 is not None) and (i2 is not None):
                                    self.indices.append(num)
                                    if not self.edges:
                                        # nothing snapped yet, we treat it
                                        # as a normal edge-snapped dimension
                                        self.node = [v1, v2]
                                        self.link = [ob, i1, i2]
                                        self.edges.append(ed)
                                        if DraftGeomUtils.geomType(ed) == "Circle":
                                            # snapped edge is an arc
                                            self.arcmode = "diameter"
                                            self.link = [ob, num]
                                    else:
                                        # there is already a snapped edge,
                                        # so we start angular dimension
                                        self.edges.append(ed)
                                        # self.node now has the 4 endpoints
                                        self.node.extend([v1, v2])
                                        c = DraftGeomUtils.findIntersection(self.node[0],
                                                                            self.node[1],
                                                                            self.node[2],
                                                                            self.node[3],
                                                                            True, True)
                                        if c:
                                            # print("centers:",c)
                                            self.center = c[0]
                                            self.arctrack.setCenter(self.center)
                                            self.arctrack.on()
                                            for e in self.edges:
                                                for v in e.Vertexes:
                                                    self.pts.append(self.arctrack.getAngle(v.Point))
                                            self.link = [self.link[0], ob]
                                        else:
                                            _msg(translate("draft", "Edges don't intersect!"))
                                            self.finish()
                                            return
                                self.dimtrack.on()
                    else:
                        self.node.append(self.point)
                    self.selectmode = False
                    # print("node", self.node)
                    self.dimtrack.update(self.node)
                    if len(self.node) == 2:
                        self.point2 = self.node[1]
                    if len(self.node) == 1:
                        self.dimtrack.on()
                        if self.planetrack:
                            self.planetrack.set(self.node[0])
                    elif len(self.node) == 2 and self.cont:
                        self.node.append(self.cont)
                        self.createObject()
                        if not self.cont:
                            self.finish()
                    elif len(self.node) == 3:
                        # for unlinked arc mode:
                        # if self.arcmode:
                        #     v = self.node[1].sub(self.node[0])
                        #     v.multiply(0.5)
                        #     cen = self.node[0].add(v)
                        #     self.node = [self.node[0], self.node[1], cen]
                        self.createObject()
                        if not self.cont:
                            self.finish()
                    elif self.angledata:
                        self.node.append(self.point)
                        self.createObject()
                        self.finish()

    def numericInput(self, numx, numy, numz):
        """Validate the entry fields in the user interface.

        This function is called by the toolbar or taskpanel interface
        when valid x, y, and z have been entered in the input fields.
        """
        self.point = App.Vector(numx, numy, numz)
        self.node.append(self.point)
        self.dimtrack.update(self.node)
        if len(self.node) == 1:
            self.dimtrack.on()
        elif len(self.node) == 3:
            self.createObject()
            if not self.cont:
                self.finish()


Gui.addCommand('Draft_Dimension', Dimension())



# -*- coding: utf-8 -*-
# AddonManager gui init module
# (c) 2001 Juergen Riegel
# License LGPL

import AddonManager
import AddonManager_rc

FreeCADGui.addLanguagePath(":/translations")
FreeCADGui.addCommand('Std_AddonMgr', AddonManager.CommandAddonManager())



"""Commands that require the graphical user interface to work.

These GUI commands are called by buttons, menus, contextual menus,
toolbars, or other ways that require graphical widgets.
They are normally loaded in the workbench's `InitGui.py`.
"""



# ***************************************************************************
# *   Copyright (c) 2017 Johannes Hartung <j.hartung@gmx.net>               *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD FEM solver Fenics tools"
__author__ = "Johannes Hartung"
__url__ = "http://www.freecadweb.org"

## @package Fenics
#  \ingroup FEM

import FreeCAD
import numpy as np

try:
    import fenics
except ImportError:
    FreeCAD.Console.PrintError("No Fenics modules found, please install them.\n")
    raise Exception("No Fenics modules found, please install them.\n")


class XDMFReader(object):
    """
    Reads XDMF file and provides unified interface for returning
    cell functions or facet functions.
    """
    def __init__(self, xdmffilename):
        """
        Sets filename and sets mesh instance to None.
        """
        self.xdmffilename = xdmffilename
        self.mesh = None

    def resetMesh(self):
        """
        Resets mesh instance to None.
        """
        self.mesh = None

    def readMesh(self):
        """
        If mesh instance is None, read mesh instance from file denoted
        by filename property.
        """
        # TODO: implement mesh read in for open file
        if self.mesh is None:
            xdmffile = fenics.XDMFFile(self.xdmffilename)
            self.mesh = fenics.Mesh()
            xdmffile.read(self.mesh)
            xdmffile.close()

    def readCellExpression(
        self,
        group_value_dict,
        value_type="scalar",
        overlap=lambda x: x[0],
        *args,
        **kwargs
    ):
        """
        Reads cell expression and returns it.
        """
        value_type_dictionary = {
            "scalar": ScalarCellExpressionFromXDMF,
            "vector2d": Vector2DCellExpressionFromXDMF,
            "vector3d": Vector3DCellExpressionFromXDMF}

        self.readMesh()
        xdmffile = fenics.XDMFFile(self.xdmffilename)
        cf = value_type_dictionary[value_type.lower()](
            group_value_dict,
            overlap=overlap,
            *args, **kwargs
        )
        cf.init()
        for (key, value) in cf.group_value_dict.items():
            cf.markers[key] = fenics.MeshFunction(
                "size_t",
                self.mesh,
                self.mesh.topology().dim()
            )
            xdmffile.read(cf.markers[key], key)
            cf.dx[key] = fenics.Measure(
                "dx",
                domain=self.mesh,
                subdomain_data=cf.markers[key]
            )
        xdmffile.close()
        return cf

    def readFacetFunction(self, group_value_dict, *args, **kwargs):
        """
        Reads facet function and returns it.
        """
        self.readMesh()
        xdmffile = fenics.XDMFFile(self.xdmffilename)
        ff = FacetFunctionFromXDMF(group_value_dict, *args, **kwargs)
        ff.init()
        for (key, value) in ff.group_value_dict.items():
            ff.markers[key] = fenics.MeshFunction(
                "size_t",
                self.mesh,
                self.mesh.topology().dim() - 1
            )
            xdmffile.read(ff.markers[key], key)
            ff.marked[key] = value.get("marked", 1)
            ff.ds[key] = fenics.Measure(
                "ds",
                domain=self.mesh,
                subdomain_data=ff.markers[key]
            )
            ff.bcs[key] = value
        xdmffile.close()
        return ff


class CellExpressionFromXDMF(object):
    """
    Creates cell function expression from XDMF file.
    """
    def __init__(
        self, group_value_dict,
        default=lambda x: 0.,
        check_marked=(lambda x: x == 1),
        overlap=lambda x: x[0],
        **kwargs
    ):
        self.init()
        self.group_value_dict = group_value_dict
        self.check_marked = check_marked
        self.overlap = overlap
        self.default = default

    def init(self):
        self.markers = {}
        self.dx = {}

    def assign_values(self, values, to_assign):
        values[:] = to_assign

    def eval_cell_backend(self, values, x, cell):

        values_list = [
            func(x) for (key, func) in self.group_value_dict.items()
            if self.check_marked(self.markers[key][cell.index])
        ]
        return_value = self.overlap(values_list)

        if values_list:
            self.assign_values(values, return_value)
        else:
            self.assign_values(values, self.default(x))

        # TODO: python classes much slower than JIT compilation


# ***********************************
# * Sub classes due to value_shape method which is not of dynamical return type
# * Also the assignment of values is to be done by reference. Therefore it has to be
# * overloaded.
# ***********************************
class ScalarCellExpressionFromXDMF(fenics.Expression, CellExpressionFromXDMF):

    def __init__(
        self,
        group_value_dict,
        default=lambda x: 0.,
        check_marked=(lambda x: x == 1),
        overlap=lambda x: x[0],
        **kwargs
    ):
        CellExpressionFromXDMF.__init__(
            self, group_value_dict,
            default=default,
            check_marked=check_marked,
            overlap=overlap
        )

    def eval_cell(self, values, x, cell):
        self.eval_cell_backend(values, x, cell)

    def value_shape(self):
        return ()


class Vector3DCellExpressionFromXDMF(fenics.Expression, CellExpressionFromXDMF):

    def __init__(
        self,
        group_value_dict,
        default=lambda x: np.zeros((3,)),
        check_marked=(lambda x: x == 1),
        overlap=lambda x: x[0], **kwargs
    ):
        CellExpressionFromXDMF.__init__(
            self, group_value_dict,
            default=default,
            check_marked=check_marked,
            overlap=overlap
        )

    def eval_cell(self, values, x, cell):
        self.eval_cell_backend(values, x, cell)

    def value_shape(self):
        return (3,)


class Vector2DCellExpressionFromXDMF(fenics.Expression, CellExpressionFromXDMF):

    def __init__(
        self,
        group_value_dict,
        default=lambda x: np.zeros((2,)),
        check_marked=(lambda x: x == 1),
        overlap=lambda x: x[0],
        **kwargs
    ):
        CellExpressionFromXDMF.__init__(
            self,
            group_value_dict,
            default=default,
            check_marked=check_marked,
            overlap=overlap
        )

    def eval_cell(self, values, x, cell):
        self.eval_cell_backend(values, x, cell)

    def value_shape(self):
        return (2,)


class FacetFunctionFromXDMF(object):
    """
    Creates facet function from XDMF file.
    """
    def __init__(self, group_value_dict, *args, **kwargs):
        self.group_value_dict = group_value_dict
        self.init()

    def init(self):
        self.markers = {}
        self.marked = {}
        self.ds = {}
        self.bcs = {}

    def getDirichletBCs(self, vectorspace, *args, **kwargs):
        dbcs = []
        for (dict_key, dict_value) in self.bcs.items():
            if dict_value["type"] == "Dirichlet":
                bc = fenics.DirichletBC(
                    vectorspace,
                    dict_value["value"],
                    self.markers[dict_key],
                    dict_value.get("marked", 1),
                    *args, **kwargs
                )
                dbcs.append(bc)
        return dbcs
    # TODO: write some functions to return integrals for Neumann and Robin
    # boundary conditions for the general case (i.e. vector, tensor)

##  @}



# -*- coding: utf-8 -*-
# ***************************************************************************
# *   Copyright (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>        *
# *   Copyright (c) 2009, 2010 Ken Cline <cline@frii.com>                   *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful,            *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with FreeCAD; if not, write to the Free Software        *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""This module provides the object code for Draft Dimension.
"""
## @package dimension
# \ingroup DRAFT
# \brief This module provides the object code for Draft Dimension.

import FreeCAD as App
import math
from PySide.QtCore import QT_TRANSLATE_NOOP
import DraftGeomUtils, DraftVecUtils
import draftutils.gui_utils as gui_utils
import draftutils.utils as utils
from draftobjects.draft_annotation import DraftAnnotation

if App.GuiUp:
    from draftviewproviders.view_dimension import ViewProviderDimensionBase
    from draftviewproviders.view_dimension import ViewProviderLinearDimension
    from draftviewproviders.view_dimension import ViewProviderAngularDimension

def make_dimension(p1,p2,p3=None,p4=None):
    """makeDimension(p1,p2,[p3]) or makeDimension(object,i1,i2,p3)
    or makeDimension(objlist,indices,p3): Creates a Dimension object with
    the dimension line passign through p3.The current line width and color
    will be used. There are multiple  ways to create a dimension, depending on
    the arguments you pass to it:
    - (p1,p2,p3): creates a standard dimension from p1 to p2
    - (object,i1,i2,p3): creates a linked dimension to the given object,
    measuring the distance between its vertices indexed i1 and i2
    - (object,i1,mode,p3): creates a linked dimension
    to the given object, i1 is the index of the (curved) edge to measure,
    and mode is either "radius" or "diameter".
    """
    if not App.ActiveDocument:
        App.Console.PrintError("No active document. Aborting\n")
        return
    obj = App.ActiveDocument.addObject("App::FeaturePython","Dimension")
    LinearDimension(obj)
    if App.GuiUp:
        ViewProviderLinearDimension(obj.ViewObject)
    if isinstance(p1,App.Vector) and isinstance(p2,App.Vector):
        obj.Start = p1
        obj.End = p2
        if not p3:
            p3 = p2.sub(p1)
            p3.multiply(0.5)
            p3 = p1.add(p3)
    elif isinstance(p2,int) and isinstance(p3,int):
        l = []
        idx = (p2,p3)
        l.append((p1,"Vertex"+str(p2+1)))
        l.append((p1,"Vertex"+str(p3+1)))
        obj.LinkedGeometry = l
        obj.Support = p1
        p3 = p4
        if not p3:
            v1 = obj.Base.Shape.Vertexes[idx[0]].Point
            v2 = obj.Base.Shape.Vertexes[idx[1]].Point
            p3 = v2.sub(v1)
            p3.multiply(0.5)
            p3 = v1.add(p3)
    elif isinstance(p3,str):
        l = []
        l.append((p1,"Edge"+str(p2+1)))
        if p3 == "radius":
            #l.append((p1,"Center"))
            if App.GuiUp:
                obj.ViewObject.Override = "R $dim"
            obj.Diameter = False
        elif p3 == "diameter":
            #l.append((p1,"Diameter"))
            if App.GuiUp:
                obj.ViewObject.Override = "Ø $dim"
            obj.Diameter = True
        obj.LinkedGeometry = l
        obj.Support = p1
        p3 = p4
        if not p3:
            p3 = p1.Shape.Edges[p2].Curve.Center.add(App.Vector(1,0,0))
    obj.Dimline = p3
    if hasattr(App,"DraftWorkingPlane"):
        normal = App.DraftWorkingPlane.axis
    else:
        normal = App.Vector(0,0,1)
    if App.GuiUp:
        # invert the normal if we are viewing it from the back
        vnorm = gui_utils.get3DView().getViewDirection()
        if vnorm.getAngle(normal) < math.pi/2:
            normal = normal.negative()
    obj.Normal = normal

    if App.GuiUp:
        gui_utils.format_object(obj)
        gui_utils.select(obj)

    return obj


def make_angular_dimension(center,angles,p3,normal=None):
    """makeAngularDimension(center,angle1,angle2,p3,[normal]): creates an angular Dimension
    from the given center, with the given list of angles, passing through p3.
    """
    if not App.ActiveDocument:
        App.Console.PrintError("No active document. Aborting\n")
        return
    obj = App.ActiveDocument.addObject("App::FeaturePython","Dimension")
    AngularDimension(obj)
    if App.GuiUp:
        ViewProviderAngularDimension(obj.ViewObject)
    obj.Center = center
    for a in range(len(angles)):
        if angles[a] > 2*math.pi:
            angles[a] = angles[a]-(2*math.pi)
    obj.FirstAngle = math.degrees(angles[1])
    obj.LastAngle = math.degrees(angles[0])
    obj.Dimline = p3
    if not normal:
        if hasattr(App,"DraftWorkingPlane"):
            normal = App.DraftWorkingPlane.axis
        else:
            normal = App.Vector(0,0,1)
    if App.GuiUp:
        # invert the normal if we are viewing it from the back
        vnorm = gui_utils.get3DView().getViewDirection()
        if vnorm.getAngle(normal) < math.pi/2:
            normal = normal.negative()

    obj.Normal = normal
            
    if App.GuiUp:
        gui_utils.format_object(obj)
        gui_utils.select(obj)

    return obj



class DimensionBase(DraftAnnotation):
    """
    The Draft Dimension Base object
    This class is not used directly, but inherited by all dimension
    objects.
    """

    def __init__(self, obj, tp = "Dimension"):
        """Add common dimension properties to the object and set them"""
        
        super(DimensionBase, self).__init__(obj, tp)

        # Draft
        obj.addProperty("App::PropertyVector",
                        "Normal",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The normal direction of this dimension"))

        obj.addProperty("App::PropertyLink",
                        "Support",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The object measured by this dimension"))

        obj.addProperty("App::PropertyLinkSubList",
                        "LinkedGeometry",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The geometry this dimension is linked to"))
                                          
        obj.addProperty("App::PropertyVectorDistance",
                        "Dimline",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "Point on which the dimension \n"
                                          "line is placed."))
                                          
        obj.Dimline = App.Vector(0,1,0)
        obj.Normal = App.Vector(0,0,1)


    def execute(self, obj):
        '''Do something when recompute object'''
        
        return


    def onChanged(self,obj,prop):
        '''Do something when a property has changed'''

        return


class LinearDimension(DimensionBase):
    """
    The Draft Linear Dimension object
    """

    def __init__(self, obj):

        super(LinearDimension, self).__init__(obj, "LinearDimension")

        obj.Proxy = self

        self.init_properties(obj)


    def init_properties(self, obj):
        """Add Linear Dimension specific properties to the object and set them"""

        # Draft
        obj.addProperty("App::PropertyVectorDistance",
                        "Start",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "Startpoint of dimension"))

        obj.addProperty("App::PropertyVectorDistance",
                        "End",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "Endpoint of dimension"))

        obj.addProperty("App::PropertyVector",
                        "Direction",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The normal direction of this dimension"))

        obj.addProperty("App::PropertyLength",
                        "Distance",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The measurement of this dimension"))

        obj.addProperty("App::PropertyBool",
                        "Diameter",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "For arc/circle measurements, false = radius, true = diameter"))

        obj.Start = App.Vector(0,0,0)
        obj.End = App.Vector(1,0,0)

    def onChanged(self,obj,prop):
        '''Do something when a property has changed'''
        if hasattr(obj, "Distance"):
            obj.setEditorMode('Distance', 1)
        #if hasattr(obj,"Normal"):
        #    obj.setEditorMode('Normal', 2)
        if hasattr(obj, "Support"):
            obj.setEditorMode('Support', 2)


    def execute(self, obj):
        """ Set start point and end point according to the linked geometry"""
        if obj.LinkedGeometry:
            if len(obj.LinkedGeometry) == 1:
                lobj = obj.LinkedGeometry[0][0]
                lsub = obj.LinkedGeometry[0][1]
                if len(lsub) == 1:
                    if "Edge" in lsub[0]:
                        n = int(lsub[0][4:])-1
                        edge = lobj.Shape.Edges[n]
                        if DraftGeomUtils.geomType(edge) == "Line":
                            obj.Start = edge.Vertexes[0].Point
                            obj.End = edge.Vertexes[-1].Point
                        elif DraftGeomUtils.geomType(edge) == "Circle":
                            c = edge.Curve.Center
                            r = edge.Curve.Radius
                            a = edge.Curve.Axis
                            ray = obj.Dimline.sub(c).projectToPlane(App.Vector(0,0,0),a)
                            if (ray.Length == 0):
                                ray = a.cross(App.Vector(1,0,0))
                                if (ray.Length == 0):
                                    ray = a.cross(App.Vector(0,1,0))
                            ray = DraftVecUtils.scaleTo(ray,r)
                            if hasattr(obj,"Diameter"):
                                if obj.Diameter:
                                    obj.Start = c.add(ray.negative())
                                    obj.End = c.add(ray)
                                else:
                                    obj.Start = c
                                    obj.End = c.add(ray)
                elif len(lsub) == 2:
                    if ("Vertex" in lsub[0]) and ("Vertex" in lsub[1]):
                        n1 = int(lsub[0][6:])-1
                        n2 = int(lsub[1][6:])-1
                        obj.Start = lobj.Shape.Vertexes[n1].Point
                        obj.End = lobj.Shape.Vertexes[n2].Point
            elif len(obj.LinkedGeometry) == 2:
                lobj1 = obj.LinkedGeometry[0][0]
                lobj2 = obj.LinkedGeometry[1][0]
                lsub1 = obj.LinkedGeometry[0][1]
                lsub2 = obj.LinkedGeometry[1][1]
                if (len(lsub1) == 1) and (len(lsub2) == 1):
                    if ("Vertex" in lsub1[0]) and ("Vertex" in lsub2[1]):
                        n1 = int(lsub1[0][6:])-1
                        n2 = int(lsub2[0][6:])-1
                        obj.Start = lobj1.Shape.Vertexes[n1].Point
                        obj.End = lobj2.Shape.Vertexes[n2].Point
        # set the distance property
        total_len = (obj.Start.sub(obj.End)).Length
        if round(obj.Distance.Value, utils.precision()) != round(total_len, utils.precision()):
            obj.Distance = total_len
        if App.GuiUp:
            if obj.ViewObject:
                obj.ViewObject.update()



class AngularDimension(DimensionBase):
    """
    The Draft AngularDimension object
    """

    def __init__(self, obj):

        super(AngularDimension, self).__init__(obj, "AngularDimension")
        
        self.init_properties(obj)

        obj.Proxy = self    


    def init_properties(self, obj):
        """Add Angular Dimension specific properties to the object and set them"""

        obj.addProperty("App::PropertyAngle",
                        "FirstAngle",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "Start angle of the dimension"))

        obj.addProperty("App::PropertyAngle",
                        "LastAngle",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "End angle of the dimension"))

        obj.addProperty("App::PropertyVectorDistance",
                        "Center",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The center point of this dimension"))

        obj.addProperty("App::PropertyAngle",
                        "Angle",
                        "Draft",
                        QT_TRANSLATE_NOOP("App::Property",
                                          "The measurement of this dimension"))

        obj.FirstAngle = 0
        obj.LastAngle = 90
        obj.Dimline = App.Vector(0,1,0)
        obj.Center = App.Vector(0,0,0)
        obj.Normal = App.Vector(0,0,1)


    def execute(self, fp):
        '''Do something when recompute object'''
        if fp.ViewObject:
            fp.ViewObject.update()


    def onChanged(self,obj,prop):
        '''Do something when a property has changed'''
        super(AngularDimension, self).onChanged(obj, prop)
        if hasattr(obj,"Angle"):
            obj.setEditorMode('Angle',1)
        if hasattr(obj,"Normal"):
            obj.setEditorMode('Normal',2)
        if hasattr(obj,"Support"):
            obj.setEditorMode('Support',2)





#***************************************************************************
#*   Copyright (c) 2016 Yorik van Havre <yorik@uncreated.net>              *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU Lesser General Public License (LGPL)    *
#*   as published by the Free Software Foundation; either version 2 of     *
#*   the License, or (at your option) any later version.                   *
#*   for detail see the LICENCE text file.                                 *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU Library General Public License for more details.                  *
#*                                                                         *
#*   You should have received a copy of the GNU Library General Public     *
#*   License along with this program; if not, write to the Free Software   *
#*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#*   USA                                                                   *
#*                                                                         *
#***************************************************************************

__title__= "FreeCAD Precast concrete module"
__author__ = "Yorik van Havre"
__url__ = "http://www.freecadweb.org"

"""This module contains tools to build basic precast concrete elements:
Beams, pillars, slabs and panels"""

import ArchCommands,ArchComponent,FreeCAD
from FreeCAD import Vector
if FreeCAD.GuiUp:
    import FreeCADGui
    from PySide import QtCore, QtGui
    from DraftTools import translate
    from PySide.QtCore import QT_TRANSLATE_NOOP
else:
    # \cond
    def translate(ctxt,txt):
        return txt
    def QT_TRANSLATE_NOOP(ctxt,txt):
        return txt
    # \endcond

## @package ArchPrecast
#  \ingroup ARCH
#  \brief Precast options for ArchStructure
#
#  This module provides additional presets for the Arch Structure
#  tool, to build a series of precast concrete elements

class _Precast(ArchComponent.Component):

    "The base Precast class"

    def __init__(self,obj):

        ArchComponent.Component.__init__(self,obj)
        _Precast.setProperties(self,obj)

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "Length" in pl:
            obj.addProperty("App::PropertyDistance","Length","Structure",QT_TRANSLATE_NOOP("App::Property","The length of this element"))
        if not "Width" in pl:
            obj.addProperty("App::PropertyDistance","Width","Structure",QT_TRANSLATE_NOOP("App::Property","The width of this element"))
        if not "Height" in pl:
            obj.addProperty("App::PropertyDistance","Height","Structure",QT_TRANSLATE_NOOP("App::Property","The height of this element"))
        if not "Nodes" in pl:
            obj.addProperty("App::PropertyVectorList","Nodes","Structure",QT_TRANSLATE_NOOP("App::Property","The structural nodes of this element"))
        self.Type = "Precast"

    def onDocumentRestored(self,obj):

        ArchComponent.Component.onDocumentRestored(self,obj)
        _Precast.setProperties(self,obj)

    def execute(self,obj):

        if self.clone(obj):
            return


class _PrecastBeam(_Precast):

    "The Precast Beam"

    def __init__(self,obj):

        _Precast.__init__(self,obj)
        self.setProperties(obj)
        obj.IfcType = "Beam"

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "Chamfer" in pl:
            obj.addProperty("App::PropertyDistance","Chamfer","Beam",QT_TRANSLATE_NOOP("App::Property","The size of the chamfer of this element"))
        if not "DentLength" in pl:
            obj.addProperty("App::PropertyDistance","DentLength","Beam",QT_TRANSLATE_NOOP("App::Property","The dent length of this element"))
        if not "DentHeight" in pl:
            obj.addProperty("App::PropertyDistance","DentHeight","Beam",QT_TRANSLATE_NOOP("App::Property","The dent height of this element"))
        if not "Dents" in pl:
            obj.addProperty("App::PropertyStringList","Dents","Beam",QT_TRANSLATE_NOOP("App::Property","The dents of this element"))

    def onDocumentRestored(self,obj):

        _Precast.onDocumentRestored(self,obj)
        self.setProperties(obj)

    def execute(self,obj):

        if self.clone(obj):
            return

        pl = obj.Placement
        length = obj.Length.Value
        width = obj.Width.Value
        height = obj.Height.Value
        chamfer = obj.Chamfer.Value
        dentlength = obj.DentLength.Value
        dentheight = obj.DentHeight.Value
        dents = obj.Dents

        if (length == 0) or (width == 0) or (height == 0):
            return
        if (chamfer >= width/2) or (chamfer >= height/2):
            return

        import Part
        p = []
        if chamfer > 0:
            p.append(Vector(0,chamfer,0))
            p.append(Vector(0,width-chamfer,0))
            p.append(Vector(0,width,chamfer))
            p.append(Vector(0,width,height-chamfer))
            p.append(Vector(0,width-chamfer,height))
            p.append(Vector(0,chamfer,height))
            p.append(Vector(0,0,height-chamfer))
            p.append(Vector(0,0,chamfer))
        else:
            p.append(Vector(0,0,0))
            p.append(Vector(0,width,0))
            p.append(Vector(0,width,height))
            p.append(Vector(0,0,height))
        p.append(p[0])
        p = Part.makePolygon(p)
        f = Part.Face(p)
        shape = f.extrude(Vector(length,0,0))
        if (dentlength > 0) and (dentheight > 0):
            p = []
            p.append(Vector(0,0,0))
            p.append(Vector(dentlength,0,0))
            p.append(Vector(dentlength,width,0))
            p.append(Vector(0,width,0))
            p.append(p[0])
            p = Part.makePolygon(p)
            f = Part.Face(p)
            d1 = f.extrude(Vector(0,0,dentheight))
            d2 = d1.copy()
            d2.translate(Vector(length-dentlength,0,0))
            shape = shape.cut(d1)
            shape = shape.cut(d2)
        for dent in dents:
            dent = dent.split(";")
            if len(dent) == 7:
                dentlength = float(dent[0])
                dentwidth = float(dent[1])
                dentheight = float(dent[2])
                dentslant = float(dent[3])
                dentchamfer = chamfer
                dentlevel = float(dent[4])
                dentrotation = float(dent[5])
                dentoffset = float(dent[6])
                if (dentlength == 0) or (dentwidth == 0) or (dentheight == 0):
                    continue
                if dentslant >= dentheight:
                    continue
                p = []
                p.append(Vector(0-dentchamfer,0,0))
                p.append(Vector(dentlength,0,dentslant))
                p.append(Vector(dentlength,0,dentheight))
                p.append(Vector(0-dentchamfer,0,dentheight))
                p.append(p[0])
                p = Part.makePolygon(p)
                f = Part.Face(p)
                dentshape = f.extrude(Vector(0,dentwidth,0))
                dentshape.rotate(Vector(0,0,0),Vector(0,0,1),dentrotation)
                if dentrotation == 0:
                    dentshape.translate(Vector(length,dentoffset,0))
                elif dentrotation == 90:
                    dentshape.translate(Vector(length-dentoffset,width,0))
                elif dentrotation == 180:
                    dentshape.translate(Vector(0,width-dentoffset,0))
                elif dentrotation == 270:
                    dentshape.translate(Vector(dentoffset,0,0))
                dentshape.translate(Vector(0,0,dentlevel))
                shape = shape.fuse(dentshape)
        shape = self.processSubShapes(obj,shape,pl)
        self.applyShape(obj,shape,pl)


class _PrecastIbeam(_Precast):

    "The Precast Ibeam"

    def __init__(self,obj):

        _Precast.__init__(self,obj)
        self.setProperties(obj)
        obj.IfcType = "Beam"

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "Chamfer" in pl:
            obj.addProperty("App::PropertyDistance","Chamfer","Beam",QT_TRANSLATE_NOOP("App::Property","The chamfer length of this element"))
        if not "BeamBase" in pl:
            obj.addProperty("App::PropertyDistance","BeamBase","Beam",QT_TRANSLATE_NOOP("App::Property","The base length of this element"))

    def onDocumentRestored(self,obj):

        _Precast.onDocumentRestored(self,obj)
        self.setProperties(obj)

    def execute(self,obj):

        if self.clone(obj):
            return

        pl = obj.Placement
        length = obj.Length.Value
        width = obj.Width.Value
        height = obj.Height.Value
        base = obj.BeamBase.Value
        slant = obj.Chamfer.Value

        if (length == 0) or (width == 0) or (height == 0):
            return
        if (slant*2 >= width) or (base*2+slant*2 >= height):
            return

        import Part
        p = []
        p.append(Vector(0,0,0))
        p.append(Vector(0,0,base))
        p.append(Vector(0,slant,base+slant))
        p.append(Vector(0,slant,height-(base+slant)))
        p.append(Vector(0,0,height-base))
        p.append(Vector(0,0,height))
        p.append(Vector(0,width,height))
        p.append(Vector(0,width,height-base))
        p.append(Vector(0,width-slant,height-(base+slant)))
        p.append(Vector(0,width-slant,base+slant))
        p.append(Vector(0,width,base))
        p.append(Vector(0,width,0))
        p.append(p[0])
        p = Part.makePolygon(p)
        f = Part.Face(p)
        shape = f.extrude(Vector(length,0,0))

        shape = self.processSubShapes(obj,shape,pl)
        self.applyShape(obj,shape,pl)


class _PrecastPillar(_Precast):

    "The Precast Pillar"

    def __init__(self,obj):

        _Precast.__init__(self,obj)
        self.setProperties(obj)
        obj.IfcType = "Column"

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "Chamfer" in pl:
            obj.addProperty("App::PropertyDistance","Chamfer","Column",QT_TRANSLATE_NOOP("App::Property","The size of the chamfer of this element"))
        if not "GrooveDepth" in pl:
            obj.addProperty("App::PropertyDistance","GrooveDepth","Column",QT_TRANSLATE_NOOP("App::Property","The groove depth of this element"))
        if not "GrooveHeight" in pl:
            obj.addProperty("App::PropertyDistance","GrooveHeight","Column",QT_TRANSLATE_NOOP("App::Property","The groove height of this element"))
        if not "GrooveSpacing" in pl:
            obj.addProperty("App::PropertyDistance","GrooveSpacing","Column",QT_TRANSLATE_NOOP("App::Property","The spacing between the grooves of this element"))
        if not "GrooveNumber" in pl:
            obj.addProperty("App::PropertyInteger","GrooveNumber","Column",QT_TRANSLATE_NOOP("App::Property","The number of grooves of this element"))
        if not "Dents" in pl:
            obj.addProperty("App::PropertyStringList","Dents","Column",QT_TRANSLATE_NOOP("App::Property","The dents of this element"))

    def onDocumentRestored(self,obj):

        _Precast.onDocumentRestored(self,obj)
        self.setProperties(obj)

    def execute(self,obj):

        if self.clone(obj):
            return

        pl = obj.Placement
        length = obj.Length.Value
        width = obj.Width.Value
        height = obj.Height.Value
        chamfer = obj.Chamfer.Value
        groovedepth = obj.GrooveDepth.Value
        grooveheight = obj.GrooveHeight.Value
        spacing = obj.GrooveSpacing.Value
        number = obj.GrooveNumber
        dents = obj.Dents

        if (length == 0) or (width == 0) or (height == 0):
            return
        if (chamfer >= width/2) or (chamfer >= length/2):
            return

        import Part
        p = []
        if chamfer > 0:
            p.append(Vector(chamfer,0,0))
            p.append(Vector(length-chamfer,0,0))
            p.append(Vector(length,chamfer,0))
            p.append(Vector(length,width-chamfer,0))
            p.append(Vector(length-chamfer,width,0))
            p.append(Vector(chamfer,width,0))
            p.append(Vector(0,width-chamfer,0))
            p.append(Vector(0,chamfer,0))
        else:
            p.append(Vector(0,0,0))
            p.append(Vector(length,0,0))
            p.append(Vector(length,width,0))
            p.append(Vector(0,width,0))
        p.append(p[0])
        p = Part.makePolygon(p)
        f = Part.Face(p)
        shape = f.extrude(Vector(0,0,height))

        if (groovedepth > 0) and (grooveheight > 0) and (spacing > 0) and (number > 0) and (groovedepth < length/2) and (groovedepth < width/2):
            p1 = []
            p1.append(Vector(0,0,0))
            p1.append(Vector(length,0,0))
            p1.append(Vector(length,width,0))
            p1.append(Vector(0,width,0))
            p1.append(p1[0])
            p1 = Part.makePolygon(p1)
            f1 = Part.Face(p1)
            groove = f1.extrude(Vector(0,0,grooveheight))
            p2 = []
            p2.append(Vector(groovedepth,groovedepth,0))
            p2.append(Vector(length-groovedepth,groovedepth,0))
            p2.append(Vector(length-groovedepth,width-groovedepth,0))
            p2.append(Vector(groovedepth,width-groovedepth,0))
            p2.append(p2[0])
            p2 = Part.makePolygon(p2)
            f2 = Part.Face(p2)
            s = f2.extrude(Vector(0,0,grooveheight))
            groove = groove.cut(s)
            for i in range(number):
                g = groove.copy()
                g.translate(Vector(0,0,spacing + i*(spacing+grooveheight)))
                shape = shape.cut(g)

        for dent in dents:
            dent = dent.split(";")
            if len(dent) == 7:
                dentlength = float(dent[0])
                dentwidth = float(dent[1])
                dentheight = float(dent[2])
                dentslant = float(dent[3])
                dentchamfer = chamfer
                dentlevel = float(dent[4])
                dentrotation = float(dent[5])
                dentoffset = float(dent[6])
                if (dentlength == 0) or (dentwidth == 0) or (dentheight == 0):
                    continue
                if dentslant >= dentheight:
                    continue
                p = []
                p.append(Vector(0-dentchamfer,0,0))
                p.append(Vector(dentlength,0,dentslant))
                p.append(Vector(dentlength,0,dentheight))
                p.append(Vector(0-dentchamfer,0,dentheight))
                p.append(p[0])
                p = Part.makePolygon(p)
                f = Part.Face(p)
                dentshape = f.extrude(Vector(0,dentwidth,0))
                dentshape.rotate(Vector(0,0,0),Vector(0,0,1),dentrotation)
                if dentrotation == 0:
                    dentshape.translate(Vector(length,dentoffset,0))
                elif dentrotation == 90:
                    dentshape.translate(Vector(length-dentoffset,width,0))
                elif dentrotation == 180:
                    dentshape.translate(Vector(0,width-dentoffset,0))
                elif dentrotation == 270:
                    dentshape.translate(Vector(dentoffset,0,0))
                dentshape.translate(Vector(0,0,dentlevel))
                shape = shape.fuse(dentshape)

        shape = self.processSubShapes(obj,shape,pl)
        self.applyShape(obj,shape,pl)


class _PrecastPanel(_Precast):

    "The Precast Panel"

    def __init__(self,obj):

        _Precast.__init__(self,obj)
        self.setProperties(obj)
        obj.IfcType = "Plate"

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "Chamfer" in pl:
            obj.addProperty("App::PropertyDistance","Chamfer","Panel",QT_TRANSLATE_NOOP("App::Property","The size of the chamfer of this element"))
        if not "DentWidth" in pl:
            obj.addProperty("App::PropertyDistance","DentWidth","Panel",QT_TRANSLATE_NOOP("App::Property","The dent width of this element"))
        if not "DentHeight" in pl:
            obj.addProperty("App::PropertyDistance","DentHeight","Panel",QT_TRANSLATE_NOOP("App::Property","The dent height of this element"))

    def onDocumentRestored(self,obj):

        _Precast.onDocumentRestored(self,obj)
        self.setProperties(obj)

    def execute(self,obj):

        if self.clone(obj):
            return

        pl = obj.Placement
        length = obj.Length.Value
        width = obj.Width.Value
        height = obj.Height.Value
        chamfer = obj.Chamfer.Value
        dentheight = obj.DentHeight.Value
        dentwidth = obj.DentWidth.Value

        if (length == 0) or (width == 0) or (height == 0):
            return
        if ((chamfer+dentwidth) >= width) or (dentheight >= height):
            return

        import Part
        p = []
        p.append(Vector(0,0,0))
        p.append(Vector(length,0,0))
        p.append(Vector(length,width,0))
        p.append(Vector(0,width,0))
        p.append(p[0])
        p = Part.makePolygon(p)
        f = Part.Face(p)
        shape = f.extrude(Vector(0,0,height))
        if chamfer > 0:
            p = []
            p.append(Vector(0,width-chamfer,0))
            p.append(Vector(chamfer,width,0))
            p.append(Vector(0,width,0))
            p.append(p[0])
            p = Part.makePolygon(p)
            f = Part.Face(p)
            s = f.extrude(Vector(0,0,height))
            shape = shape.cut(s)
            p = []
            p.append(Vector(length,width-chamfer,0))
            p.append(Vector(length-chamfer,width,0))
            p.append(Vector(length,width,0))
            p.append(p[0])
            p = Part.makePolygon(p)
            f = Part.Face(p)
            s = f.extrude(Vector(0,0,height))
            shape = shape.cut(s)
            p = []
            p.append(Vector(0,width-chamfer,0))
            p.append(Vector(0,width,chamfer))
            p.append(Vector(0,width,0))
            p.append(p[0])
            p = Part.makePolygon(p)
            f = Part.Face(p)
            s = f.extrude(Vector(length,0,0))
            shape = shape.cut(s)
            p = []
            p.append(Vector(0,width-chamfer,height))
            p.append(Vector(0,width,height-chamfer))
            p.append(Vector(0,width,height))
            p.append(p[0])
            p = Part.makePolygon(p)
            f = Part.Face(p)
            s = f.extrude(Vector(length,0,0))
            shape = shape.cut(s)
        if (dentheight > 0) and (dentwidth > 0):
            p = []
            p.append(Vector(0,((width-chamfer)-dentwidth)/2,0))
            p.append(Vector(0,((width-chamfer)-dentwidth)/2+dentwidth,0))
            p.append(Vector(0,((width-chamfer)-dentwidth)/2+dentwidth,dentheight))
            p.append(Vector(0,((width-chamfer)-dentwidth)/2,dentheight))
            p.append(p[0])
            p = Part.makePolygon(p)
            f = Part.Face(p)
            s = f.extrude(Vector(length,0,0))
            shape = shape.cut(s)
            s.translate(Vector(0,0,height))
            shape = shape.fuse(s)

        shape = self.processSubShapes(obj,shape,pl)
        self.applyShape(obj,shape,pl)


class _PrecastSlab(_Precast):

    "The Precast Slab"

    def __init__(self,obj):

        _Precast.__init__(self,obj)
        self.setProperties(obj)
        obj.IfcType = "Slab"

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "SlabType" in pl:
            obj.addProperty("App::PropertyEnumeration","SlabType","Slab",QT_TRANSLATE_NOOP("App::Property","The type of this slab"))
            obj.SlabType = ["Champagne","Hat"]
        if not "SlabBase" in pl:
            obj.addProperty("App::PropertyDistance","SlabBase","Slab",QT_TRANSLATE_NOOP("App::Property","The size of the base of this element"))
        if not "HoleNumber" in pl:
            obj.addProperty("App::PropertyInteger","HoleNumber","Slab",QT_TRANSLATE_NOOP("App::Property","The number of holes in this element"))
        if not "HoleMajor" in pl:
            obj.addProperty("App::PropertyDistance","HoleMajor","Slab",QT_TRANSLATE_NOOP("App::Property","The major radius of the holes of this element"))
        if not "HoleMinor" in pl:
            obj.addProperty("App::PropertyDistance","HoleMinor","Slab",QT_TRANSLATE_NOOP("App::Property","The minor radius of the holes of this element"))
        if not "HoleSpacing" in pl:
            obj.addProperty("App::PropertyDistance","HoleSpacing","Slab",QT_TRANSLATE_NOOP("App::Property","The spacing between the holes of this element"))

    def onDocumentRestored(self,obj):

        _Precast.onDocumentRestored(self,obj)
        self.setProperties(obj)

    def execute(self,obj):

        if self.clone(obj):
            return

        pl = obj.Placement
        slabtype = obj.SlabType
        length = obj.Length.Value
        width = obj.Width.Value
        height = obj.Height.Value
        base = obj.SlabBase.Value
        holenumber = obj.HoleNumber
        holemajor = obj.HoleMajor.Value
        holeminor = obj.HoleMinor.Value
        holespacing = obj.HoleSpacing.Value

        slant = (height-base) / 3 # this gives the inclination of the vertical walls

        if (length == 0) or (width == 0) or (height == 0):
            return
        if base >= height:
            return
        if height < (base*2):
            return
        if (holenumber > 0) and ( (holespacing == 0) or (holemajor == 0) or (holeminor == 0) ):
            return
        if holemajor < holeminor:
            return
        import Part

        p = []
        if slabtype == "Champagne":
            p.append(Vector(0,0,0))
            p.append(Vector(0,slant,height-base))
            p.append(Vector(0,0,height-base))
            p.append(Vector(0,0,height))
            p.append(Vector(0,width,height))
            p.append(Vector(0,width,height-base))
            p.append(Vector(0,width-slant,height-base))
            p.append(Vector(0,width,0))
        elif slabtype == "Hat":
            p.append(Vector(0,0,0))
            p.append(Vector(0,0,base))
            p.append(Vector(0,slant,base))
            p.append(Vector(0,slant*2,height))
            p.append(Vector(0,width-slant*2,height))
            p.append(Vector(0,width-slant,base))
            p.append(Vector(0,width,base))
            p.append(Vector(0,width,0))
        else:
            return None
        p.append(p[0])
        p = Part.makePolygon(p)
        f = Part.Face(p)
        shape = f.extrude(Vector(length,0,0))

        if holenumber > 0:
            holespan = holenumber * holeminor + (holenumber - 1) * holespacing
            holestart = (width/2 - holespan/2) + holeminor/2
            if holeminor != holemajor:
                e = Part.Ellipse(Vector(0,0,0),Vector(0,holeminor/2,0),Vector(0,0,holemajor/2)).toShape()
                e.translate(Vector(0,0,-holemajor/2))
            else:
                e = Part.Circle(Vector(0,0,0),Vector(1,0,0),holemajor/2).toShape()
            w = Part.Wire([e])
            f = Part.Face(w)
            tube = f.extrude(Vector(length,0,0))
            for i in range(holenumber):
                x = holestart + i*(holeminor+holespacing)
                s = tube.copy()
                s.translate(Vector(0,x,height/2))
                shape = shape.cut(s)

        shape = self.processSubShapes(obj,shape,pl)
        self.applyShape(obj,shape,pl)


class _PrecastStairs(_Precast):

    "The Precast Stairs"

    def __init__(self,obj):

        _Precast.__init__(self,obj)
        self.setProperties(obj)
        obj.IfcType = "Stair"

    def setProperties(self,obj):

        pl = obj.PropertiesList
        if not "DownLength" in pl:
            obj.addProperty("App::PropertyDistance","DownLength","Stairs",QT_TRANSLATE_NOOP("App::Property","The length of the down floor of this element"))
        if not "RiserNumber" in pl:
            obj.addProperty("App::PropertyInteger","RiserNumber","Stairs",QT_TRANSLATE_NOOP("App::Property","The number of risers in this element"))
        if not "Riser" in pl:
            obj.addProperty("App::PropertyDistance","Riser","Stairs",QT_TRANSLATE_NOOP("App::Property","The riser height of this element"))
        if not "Tread" in pl:
            obj.addProperty("App::PropertyDistance","Tread","Stairs",QT_TRANSLATE_NOOP("App::Property","The tread depth of this element"))

    def onDocumentRestored(self,obj):

        _Precast.onDocumentRestored(self,obj)
        self.setProperties(obj)

    def execute(self,obj):

        if self.clone(obj):
            return

        pl = obj.Placement
        length = obj.Length.Value
        width = obj.Width.Value
        height = obj.Height.Value
        downlength = obj.DownLength.Value
        steps = obj.RiserNumber
        riser = obj.Riser.Value
        tread = obj.Tread.Value

        if not width:
            return
        if not steps:
            return
        if not riser:
            return
        if not tread:
            return
        if not height:
            return
        if length < tread:
            length = tread # minimum

        import math,Part

        p = [Vector(0,0,0)] # relative moves
        if downlength:
            p.append(Vector(0,downlength,0))
        for i in range(steps-1):
            p.append(Vector(0,0,riser))
            p.append(Vector(0,tread,0))
        p.append(Vector(0,0,riser))
        p.append(Vector(0,length,0))
        ang1 = math.atan(riser/tread)
        ang2 = ang1/2
        rdist = math.tan(ang2)*height
        if length > (tread+rdist):
            p.append(Vector(0,0,-height))
            p.append(Vector(0,-(length-(tread+rdist))))
        else:
            rest = length-(tread+rdist)
            addh = math.tan(ang1)*rest
            p.append(Vector(0,0,-(height+addh)))
        # absolutize
        r = [p[0]]
        for m in p[1:]:
            r.append(r[-1].add(m))
        p = r
        if downlength:
            bdist = math.tan(ang1)*height
            p.append(Vector(0,downlength+bdist,-height))
            p.append(Vector(0,0,-height))
        else:
            hdist = height*(math.tan(ang1))
            p.append(Vector(0,hdist,0))
        p.append(p[0])
        p = Part.makePolygon(p)
        f = Part.Face(p)
        shape = f.extrude(Vector(width,0,0))

        shape = self.processSubShapes(obj,shape,pl)
        self.applyShape(obj,shape,pl)


class _ViewProviderPrecast(ArchComponent.ViewProviderComponent):

    "The View Provider of the Precast object"

    def __init__(self,vobj):

        ArchComponent.ViewProviderComponent.__init__(self,vobj)
        vobj.ShapeColor = ArchCommands.getDefaultColor("Structure")

    def getIcon(self):

        import Arch_rc
        if hasattr(self,"Object"):
            if self.Object.CloneOf:
                return ":/icons/Arch_Structure_Clone.svg"
        return ":/icons/Arch_Structure_Tree.svg"

    def setEdit(self,vobj,mode):

        if mode == 0:
            import FreeCADGui
            taskd = ArchComponent.ComponentTaskPanel()
            taskd.obj = self.Object
            taskd.update()
            if hasattr(self.Object,"Dents"):
                self.dentd = _DentsTaskPanel()
                self.dentd.form.show()
                self.dentd.fillDents(self.Object.Dents)
                taskd.form = [taskd.form,self.dentd.form]
            FreeCADGui.Control.showDialog(taskd)
            return True
        return False

    def unsetEdit(self,vobj,mode):

        import FreeCADGui
        if hasattr(self,"dentd"):
            self.Object.Dents = self.dentd.getValues()
            del self.dentd
        FreeCADGui.Control.closeDialog()
        return False


class _PrecastTaskPanel:

    '''The TaskPanel for precast creation'''

    def __init__(self):

        import FreeCADGui
        from PySide import QtCore,QtGui,QtSvg
        self.form = QtGui.QWidget()
        self.grid = QtGui.QGridLayout(self.form)
        self.PrecastTypes = ["Beam","I-Beam","Pillar","Panel","Slab","Stairs"]
        self.SlabTypes = ["Champagne","Hat"]

        # image display
        self.preview = QtSvg.QSvgWidget(":/ui/ParametersBeam.svg")
        self.preview.setMaximumWidth(200)
        self.preview.setMinimumHeight(120)
        self.grid.addWidget(self.preview,0,0,1,2)

        # parameters
        self.labelSlabType = QtGui.QLabel()
        self.valueSlabType = QtGui.QComboBox()
        self.valueSlabType.addItems(self.SlabTypes)
        self.valueSlabType.setCurrentIndex(0)
        self.grid.addWidget(self.labelSlabType,1,0,1,1)
        self.grid.addWidget(self.valueSlabType,1,1,1,1)

        self.labelChamfer = QtGui.QLabel()
        self.valueChamfer = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelChamfer,2,0,1,1)
        self.grid.addWidget(self.valueChamfer,2,1,1,1)

        self.labelDentLength = QtGui.QLabel()
        self.valueDentLength = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelDentLength,3,0,1,1)
        self.grid.addWidget(self.valueDentLength,3,1,1,1)

        self.labelDentWidth = QtGui.QLabel()
        self.valueDentWidth = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelDentWidth,4,0,1,1)
        self.grid.addWidget(self.valueDentWidth,4,1,1,1)

        self.labelDentHeight = QtGui.QLabel()
        self.valueDentHeight = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelDentHeight,5,0,1,1)
        self.grid.addWidget(self.valueDentHeight,5,1,1,1)

        self.labelBase = QtGui.QLabel()
        self.valueBase = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelBase,6,0,1,1)
        self.grid.addWidget(self.valueBase,6,1,1,1)

        self.labelHoleNumber = QtGui.QLabel()
        self.valueHoleNumber = QtGui.QSpinBox()
        self.grid.addWidget(self.labelHoleNumber,7,0,1,1)
        self.grid.addWidget(self.valueHoleNumber,7,1,1,1)

        self.labelHoleMajor = QtGui.QLabel()
        self.valueHoleMajor = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelHoleMajor,8,0,1,1)
        self.grid.addWidget(self.valueHoleMajor,8,1,1,1)

        self.labelHoleMinor = QtGui.QLabel()
        self.valueHoleMinor = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelHoleMinor,9,0,1,1)
        self.grid.addWidget(self.valueHoleMinor,9,1,1,1)

        self.labelHoleSpacing = QtGui.QLabel()
        self.valueHoleSpacing = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelHoleSpacing,10,0,1,1)
        self.grid.addWidget(self.valueHoleSpacing,10,1,1,1)

        self.labelGrooveNumber = QtGui.QLabel()
        self.valueGrooveNumber = QtGui.QSpinBox()
        self.grid.addWidget(self.labelGrooveNumber,11,0,1,1)
        self.grid.addWidget(self.valueGrooveNumber,11,1,1,1)

        self.labelGrooveDepth = QtGui.QLabel()
        self.valueGrooveDepth = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelGrooveDepth,12,0,1,1)
        self.grid.addWidget(self.valueGrooveDepth,12,1,1,1)

        self.labelGrooveHeight = QtGui.QLabel()
        self.valueGrooveHeight = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelGrooveHeight,13,0,1,1)
        self.grid.addWidget(self.valueGrooveHeight,13,1,1,1)

        self.labelGrooveSpacing = QtGui.QLabel()
        self.valueGrooveSpacing = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelGrooveSpacing,14,0,1,1)
        self.grid.addWidget(self.valueGrooveSpacing,14,1,1,1)

        self.labelRiserNumber = QtGui.QLabel()
        self.valueRiserNumber = QtGui.QSpinBox()
        self.grid.addWidget(self.labelRiserNumber,15,0,1,1)
        self.grid.addWidget(self.valueRiserNumber,15,1,1,1)

        self.labelDownLength = QtGui.QLabel()
        self.valueDownLength = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelDownLength,16,0,1,1)
        self.grid.addWidget(self.valueDownLength,16,1,1,1)

        self.labelRiser = QtGui.QLabel()
        self.valueRiser = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelRiser,17,0,1,1)
        self.grid.addWidget(self.valueRiser,17,1,1,1)

        self.labelTread = QtGui.QLabel()
        self.valueTread = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelTread,18,0,1,1)
        self.grid.addWidget(self.valueTread,18,1,1,1)
        
        # signals/slots
        QtCore.QObject.connect(self.valueChamfer,QtCore.SIGNAL("valueChanged(double)"),self.setChamfer)
        QtCore.QObject.connect(self.valueDentLength,QtCore.SIGNAL("valueChanged(double)"),self.setDentLength)
        QtCore.QObject.connect(self.valueDentWidth,QtCore.SIGNAL("valueChanged(double)"),self.setDentWidth)
        QtCore.QObject.connect(self.valueDentHeight,QtCore.SIGNAL("valueChanged(double)"),self.setDentHeight)
        QtCore.QObject.connect(self.valueBase,QtCore.SIGNAL("valueChanged(double)"),self.setBase)
        QtCore.QObject.connect(self.valueHoleMajor,QtCore.SIGNAL("valueChanged(double)"),self.setHoleMajor)
        QtCore.QObject.connect(self.valueHoleMinor,QtCore.SIGNAL("valueChanged(double)"),self.setHoleMinor)
        QtCore.QObject.connect(self.valueHoleSpacing,QtCore.SIGNAL("valueChanged(double)"),self.setHoleSpacing)
        QtCore.QObject.connect(self.valueGrooveDepth,QtCore.SIGNAL("valueChanged(double)"),self.setGrooveDepth)
        QtCore.QObject.connect(self.valueGrooveHeight,QtCore.SIGNAL("valueChanged(double)"),self.setGrooveHeight)
        QtCore.QObject.connect(self.valueGrooveSpacing,QtCore.SIGNAL("valueChanged(double)"),self.setGrooveSpacing)
        QtCore.QObject.connect(self.valueDownLength,QtCore.SIGNAL("valueChanged(double)"),self.setDownLength)
        QtCore.QObject.connect(self.valueRiser,QtCore.SIGNAL("valueChanged(double)"),self.setRiser)
        QtCore.QObject.connect(self.valueTread,QtCore.SIGNAL("valueChanged(double)"),self.setTread)

        # restore presets
        param = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch")
        self.valueChamfer.setText(str(param.GetFloat("PrecastChamfer",0.0)))
        self.valueDentLength.setText(str(param.GetFloat("PrecastDentLength",0.0)))
        self.valueDentWidth.setText(str(param.GetFloat("PrecastDentWidth",0.0)))
        self.valueDentHeight.setText(str(param.GetFloat("PrecastDentHeight",0.0)))
        self.valueBase.setText(str(param.GetFloat("PrecastBase",0.0)))
        self.valueHoleMajor.setText(str(param.GetFloat("PrecastHoleMajor",0.0)))
        self.valueHoleMinor.setText(str(param.GetFloat("PrecastHoleMinor",0.0)))
        self.valueHoleSpacing.setText(str(param.GetFloat("PrecastHoleSpacing",0.0)))
        self.valueGrooveDepth.setText(str(param.GetFloat("PrecastGrooveDepth",0.0)))
        self.valueGrooveHeight.setText(str(param.GetFloat("PrecastGrooveHeight",0.0)))
        self.valueGrooveSpacing.setText(str(param.GetFloat("PrecastGrooveSpacing",0.0)))
        self.valueDownLength.setText(str(param.GetFloat("PrecastDownLength",0.0)))
        self.valueRiser.setText(str(param.GetFloat("PrecastRiser",0.0)))
        self.valueTread.setText(str(param.GetFloat("PrecastTread",0.0)))

        self.retranslateUi(self.form)
        self.form.hide()

    def getValues(self):
        d = {}
        d["SlabType"] = self.SlabTypes[self.valueSlabType.currentIndex()]
        d["Chamfer"] = self.Chamfer
        d["DentLength"] = self.DentLength
        d["DentWidth"] = self.DentWidth
        d["DentHeight"] = self.DentHeight
        d["Base"] = self.Base
        d["HoleNumber"] = self.valueHoleNumber.value()
        d["HoleMajor"] = self.HoleMajor
        d["HoleMinor"] = self.HoleMinor
        d["HoleSpacing"] = self.HoleSpacing
        d["GrooveNumber"] = self.valueGrooveNumber.value()
        d["GrooveDepth"] = self.GrooveDepth
        d["GrooveHeight"] = self.GrooveHeight
        d["GrooveSpacing"] = self.GrooveSpacing
        d["RiserNumber"] = self.valueRiserNumber.value()
        d["DownLength"] = self.DownLength
        d["Riser"] = self.Riser
        d["Tread"] = self.Tread
        if hasattr(self,"Dents"):
            d["Dents"] = self.Dents.getValues()
        return d

    def setChamfer(self,value):
        self.Chamfer = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastChamfer",value)

    def setDentLength(self,value):
        self.DentLength = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastDentLength",value)

    def setDentWidth(self,value):
        self.DentWidth = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastDentWidth",value)

    def setDentHeight(self,value):
        self.DentHeight = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastDentHeight",value)

    def setBase(self,value):
        self.Base = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastBase",value)

    def setHoleMajor(self,value):
        self.HoleMajor = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastHoleMajor",value)

    def setHoleMinor(self,value):
        self.HoleMinor = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastHoleMinor",value)

    def setHoleSpacing(self,value):
        self.HoleSpacing = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastHoleSpacing",value)

    def setGrooveDepth(self,value):
        self.GrooveDepth = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastGrooveDepth",value)

    def setGrooveHeight(self,value):
        self.GrooveHeight = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastGrooveHeight",value)

    def setGrooveSpacing(self,value):
        self.GrooveSpacing = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastGrooveSpacing",value)

    def setDownLength(self,value):
        self.DownLength = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastDownLength",value)

    def setRiser(self,value):
        self.Riser = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastRiser",value)

    def setTread(self,value):
        self.Tread = value
        FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Arch").SetFloat("PrecastTread",value)

    def retranslateUi(self, dialog):
        from PySide import QtGui
        self.form.setWindowTitle(translate("Arch", "Precast elements"))
        self.labelSlabType.setText(translate("Arch", "Slab type"))
        self.labelChamfer.setText(translate("Arch", "Chamfer"))
        self.labelDentLength.setText(translate("Arch", "Dent length"))
        self.labelDentWidth.setText(translate("Arch", "Dent width"))
        self.labelDentHeight.setText(translate("Arch", "Dent height"))
        self.labelBase.setText(translate("Arch", "Slab base"))
        self.labelHoleNumber.setText(translate("Arch", "Number of holes"))
        self.labelHoleMajor.setText(translate("Arch", "Major diameter of holes"))
        self.labelHoleMinor.setText(translate("Arch", "Minor diameter of holes"))
        self.labelHoleSpacing.setText(translate("Arch", "Spacing between holes"))
        self.labelGrooveNumber.setText(translate("Arch", "Number of grooves"))
        self.labelGrooveDepth.setText(translate("Arch", "Depth of grooves"))
        self.labelGrooveHeight.setText(translate("Arch", "Height of grooves"))
        self.labelGrooveSpacing.setText(translate("Arch", "Spacing between grooves"))
        self.labelRiserNumber.setText(translate("Arch", "Number of risers"))
        self.labelDownLength.setText(translate("Arch", "Length of down floor"))
        self.labelRiser.setText(translate("Arch", "Height of risers"))
        self.labelTread.setText(translate("Arch", "Depth of treads"))

    def setPreset(self,preset):
        self.preview.hide()
        if preset == "Beam":
            self.preview.load(":/ui/ParametersBeam.svg")
            self.labelSlabType.hide()
            self.valueSlabType.hide()
            self.labelChamfer.show()
            self.valueChamfer.show()
            self.labelDentLength.show()
            self.valueDentLength.show()
            self.labelDentWidth.hide()
            self.valueDentWidth.hide()
            self.labelDentHeight.show()
            self.valueDentHeight.show()
            self.labelBase.hide()
            self.valueBase.hide()
            self.labelHoleNumber.hide()
            self.valueHoleNumber.hide()
            self.labelHoleMajor.hide()
            self.valueHoleMajor.hide()
            self.labelHoleMinor.hide()
            self.valueHoleMinor.hide()
            self.labelHoleSpacing.hide()
            self.valueHoleSpacing.hide()
            self.labelGrooveNumber.hide()
            self.valueGrooveNumber.hide()
            self.labelGrooveDepth.hide()
            self.valueGrooveDepth.hide()
            self.labelGrooveHeight.hide()
            self.valueGrooveHeight.hide()
            self.labelGrooveSpacing.hide()
            self.valueGrooveSpacing.hide()
            self.valueHoleSpacing.hide()
            self.labelRiserNumber.hide()
            self.valueRiserNumber.hide()
            self.labelDownLength.hide()
            self.valueDownLength.hide()
            self.labelRiser.hide()
            self.valueRiser.hide()
            self.labelTread.hide()
            self.valueTread.hide()
        elif preset == "Pillar":
            self.preview.load(":/ui/ParametersPillar.svg")
            self.labelSlabType.hide()
            self.valueSlabType.hide()
            self.labelChamfer.show()
            self.valueChamfer.show()
            self.labelDentLength.hide()
            self.valueDentLength.hide()
            self.labelDentWidth.hide()
            self.valueDentWidth.hide()
            self.labelDentHeight.hide()
            self.valueDentHeight.hide()
            self.labelBase.hide()
            self.valueBase.hide()
            self.labelHoleNumber.hide()
            self.valueHoleNumber.hide()
            self.labelHoleMajor.hide()
            self.valueHoleMajor.hide()
            self.labelHoleMinor.hide()
            self.valueHoleMinor.hide()
            self.labelHoleSpacing.hide()
            self.valueHoleSpacing.hide()
            self.labelGrooveNumber.show()
            self.valueGrooveNumber.show()
            self.labelGrooveDepth.show()
            self.valueGrooveDepth.show()
            self.labelGrooveHeight.show()
            self.valueGrooveHeight.show()
            self.labelGrooveSpacing.show()
            self.valueGrooveSpacing.show()
            self.labelRiserNumber.hide()
            self.valueRiserNumber.hide()
            self.labelDownLength.hide()
            self.valueDownLength.hide()
            self.labelRiser.hide()
            self.valueRiser.hide()
            self.labelTread.hide()
            self.valueTread.hide()
        elif preset == "Panel":
            self.preview.load(":/ui/ParametersPanel.svg")
            self.labelSlabType.hide()
            self.valueSlabType.hide()
            self.labelChamfer.show()
            self.valueChamfer.show()
            self.labelDentLength.hide()
            self.valueDentLength.hide()
            self.labelDentWidth.show()
            self.valueDentWidth.show()
            self.labelDentHeight.show()
            self.valueDentHeight.show()
            self.labelBase.hide()
            self.valueBase.hide()
            self.labelHoleNumber.hide()
            self.valueHoleNumber.hide()
            self.labelHoleMajor.hide()
            self.valueHoleMajor.hide()
            self.labelHoleMinor.hide()
            self.valueHoleMinor.hide()
            self.labelHoleSpacing.hide()
            self.valueHoleSpacing.hide()
            self.labelGrooveNumber.hide()
            self.valueGrooveNumber.hide()
            self.labelGrooveDepth.hide()
            self.valueGrooveDepth.hide()
            self.labelGrooveHeight.hide()
            self.valueGrooveHeight.hide()
            self.labelGrooveSpacing.hide()
            self.valueGrooveSpacing.hide()
            self.labelRiserNumber.hide()
            self.valueRiserNumber.hide()
            self.labelDownLength.hide()
            self.valueDownLength.hide()
            self.labelRiser.hide()
            self.valueRiser.hide()
            self.labelTread.hide()
            self.valueTread.hide()
        elif preset == "Slab":
            self.preview.load(":/ui/ParametersSlab.svg")
            self.labelSlabType.show()
            self.valueSlabType.show()
            self.labelChamfer.hide()
            self.valueChamfer.hide()
            self.labelDentLength.hide()
            self.valueDentLength.hide()
            self.labelDentWidth.hide()
            self.valueDentWidth.hide()
            self.labelDentHeight.hide()
            self.valueDentHeight.hide()
            self.labelBase.show()
            self.valueBase.show()
            self.labelHoleNumber.show()
            self.valueHoleNumber.show()
            self.labelHoleMajor.show()
            self.valueHoleMajor.show()
            self.labelHoleMinor.show()
            self.valueHoleMinor.show()
            self.labelHoleSpacing.show()
            self.valueHoleSpacing.show()
            self.labelGrooveNumber.hide()
            self.valueGrooveNumber.hide()
            self.labelGrooveDepth.hide()
            self.valueGrooveDepth.hide()
            self.labelGrooveHeight.hide()
            self.valueGrooveHeight.hide()
            self.labelGrooveSpacing.hide()
            self.valueGrooveSpacing.hide()
            self.labelRiserNumber.hide()
            self.valueRiserNumber.hide()
            self.labelDownLength.hide()
            self.valueDownLength.hide()
            self.labelRiser.hide()
            self.valueRiser.hide()
            self.labelTread.hide()
            self.valueTread.hide()
        elif preset == "I-Beam":
            self.preview.load(":/ui/ParametersIbeam.svg")
            self.labelSlabType.hide()
            self.valueSlabType.hide()
            self.labelChamfer.show()
            self.valueChamfer.show()
            self.labelDentLength.hide()
            self.valueDentLength.hide()
            self.labelDentWidth.hide()
            self.valueDentWidth.hide()
            self.labelDentHeight.hide()
            self.valueDentHeight.hide()
            self.labelBase.show()
            self.valueBase.show()
            self.labelHoleNumber.hide()
            self.valueHoleNumber.hide()
            self.labelHoleMajor.hide()
            self.valueHoleMajor.hide()
            self.labelHoleMinor.hide()
            self.valueHoleMinor.hide()
            self.labelHoleSpacing.hide()
            self.valueHoleSpacing.hide()
            self.labelGrooveNumber.hide()
            self.valueGrooveNumber.hide()
            self.labelGrooveDepth.hide()
            self.valueGrooveDepth.hide()
            self.labelGrooveHeight.hide()
            self.valueGrooveHeight.hide()
            self.labelGrooveSpacing.hide()
            self.valueGrooveSpacing.hide()
            self.labelRiserNumber.hide()
            self.valueRiserNumber.hide()
            self.labelDownLength.hide()
            self.valueDownLength.hide()
            self.labelRiser.hide()
            self.valueRiser.hide()
            self.labelTread.hide()
            self.valueTread.hide()
        elif preset == "Stairs":
            self.preview.load(":/ui/ParametersStairs.svg")
            self.labelSlabType.hide()
            self.valueSlabType.hide()
            self.labelChamfer.hide()
            self.valueChamfer.hide()
            self.labelDentLength.hide()
            self.valueDentLength.hide()
            self.labelDentWidth.hide()
            self.valueDentWidth.hide()
            self.labelDentHeight.hide()
            self.valueDentHeight.hide()
            self.labelBase.hide()
            self.valueBase.hide()
            self.labelHoleNumber.hide()
            self.valueHoleNumber.hide()
            self.labelHoleMajor.hide()
            self.valueHoleMajor.hide()
            self.labelHoleMinor.hide()
            self.valueHoleMinor.hide()
            self.labelHoleSpacing.hide()
            self.valueHoleSpacing.hide()
            self.labelGrooveNumber.hide()
            self.valueGrooveNumber.hide()
            self.labelGrooveDepth.hide()
            self.valueGrooveDepth.hide()
            self.labelGrooveHeight.hide()
            self.valueGrooveHeight.hide()
            self.labelGrooveSpacing.hide()
            self.valueGrooveSpacing.hide()
            self.labelRiserNumber.show()
            self.valueRiserNumber.show()
            self.labelDownLength.show()
            self.valueDownLength.show()
            self.labelRiser.show()
            self.valueRiser.show()
            self.labelTread.show()
            self.valueTread.show()
        self.preview.show()


class _DentsTaskPanel:

    '''The TaskPanel for dent creation'''

    def __init__(self):

        import FreeCADGui
        from PySide import QtCore,QtGui,QtSvg
        self.form = QtGui.QWidget()
        self.grid = QtGui.QGridLayout(self.form)
        self.Rotations = ["N","S","E","O"]
        self.RotationAngles = [90,270,0,180]

        # dents list
        self.labelDents = QtGui.QLabel()
        self.listDents = QtGui.QListWidget()
        self.grid.addWidget(self.labelDents,0,0,1,2)
        self.grid.addWidget(self.listDents,1,0,1,2)

        # buttons
        self.buttonAdd = QtGui.QPushButton()
        self.buttonRemove = QtGui.QPushButton()
        self.grid.addWidget(self.buttonAdd,2,0,1,1)
        self.grid.addWidget(self.buttonRemove,2,1,1,1)

        # image display
        self.preview = QtSvg.QSvgWidget(":/ui/ParametersDent.svg")
        self.preview.setMaximumWidth(200)
        self.preview.setMinimumHeight(120)
        self.grid.addWidget(self.preview,3,0,1,2)

        # parameters
        self.labelLength = QtGui.QLabel()
        self.valueLength = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelLength,4,0,1,1)
        self.grid.addWidget(self.valueLength,4,1,1,1)

        self.labelWidth = QtGui.QLabel()
        self.valueWidth = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelWidth,5,0,1,1)
        self.grid.addWidget(self.valueWidth,5,1,1,1)

        self.labelHeight = QtGui.QLabel()
        self.valueHeight = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelHeight,6,0,1,1)
        self.grid.addWidget(self.valueHeight,6,1,1,1)

        self.labelSlant = QtGui.QLabel()
        self.valueSlant = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelSlant,7,0,1,1)
        self.grid.addWidget(self.valueSlant,7,1,1,1)

        self.labelLevel = QtGui.QLabel()
        self.valueLevel = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelLevel,8,0,1,1)
        self.grid.addWidget(self.valueLevel,8,1,1,1)

        self.labelRotation = QtGui.QLabel()
        self.valueRotation = QtGui.QComboBox()
        self.valueRotation.addItems(self.Rotations)
        self.valueRotation.setCurrentIndex(0)
        self.grid.addWidget(self.labelRotation,9,0,1,1)
        self.grid.addWidget(self.valueRotation,9,1,1,1)

        self.labelOffset = QtGui.QLabel()
        self.valueOffset = FreeCADGui.UiLoader().createWidget("Gui::InputField")
        self.grid.addWidget(self.labelOffset,10,0,1,1)
        self.grid.addWidget(self.valueOffset,10,1,1,1)

        # signals/slots
        QtCore.QObject.connect(self.valueLength,QtCore.SIGNAL("valueChanged(double)"),self.setLength)
        QtCore.QObject.connect(self.valueWidth,QtCore.SIGNAL("valueChanged(double)"),self.setWidth)
        QtCore.QObject.connect(self.valueHeight,QtCore.SIGNAL("valueChanged(double)"),self.setHeight)
        QtCore.QObject.connect(self.valueSlant,QtCore.SIGNAL("valueChanged(double)"),self.setSlant)
        QtCore.QObject.connect(self.valueLevel,QtCore.SIGNAL("valueChanged(double)"),self.setLevel)
        QtCore.QObject.connect(self.valueRotation,QtCore.SIGNAL("currentIndexChanged(int)"),self.setDent)
        QtCore.QObject.connect(self.valueOffset,QtCore.SIGNAL("valueChanged(double)"),self.setOffset)
        QtCore.QObject.connect(self.buttonAdd,QtCore.SIGNAL("clicked()"),self.addDent)
        QtCore.QObject.connect(self.buttonRemove,QtCore.SIGNAL("clicked()"),self.removeDent)
        QtCore.QObject.connect(self.listDents,QtCore.SIGNAL("itemClicked(QListWidgetItem*)"),self.editDent)
        self.retranslateUi(self.form)
        self.form.hide()

    def setLength(self,value):
        self.Length = value
        self.setDent()

    def setWidth(self,value):
        self.Width = value
        self.setDent()

    def setHeight(self,value):
        self.Height = value
        self.setDent()

    def setSlant(self,value):
        self.Slant = value
        self.setDent()

    def setLevel(self,value):
        self.Level = value
        self.setDent()

    def setOffset(self,value):
        self.Offset = value
        self.setDent()

    def fillDents(self,dents):
        self.listDents.clear()
        i = 1
        for d in dents:
            s = "Dent "+str(i)+" :"+d
            self.listDents.addItem(s)
            i += 1

    def setDent(self,i=0):
        if self.listDents.currentItem():
            num = str(self.listDents.currentRow()+1)
            rot = self.RotationAngles[self.valueRotation.currentIndex()]
            s = "Dent "+num+" :"+str(self.Length)+";"+str(self.Width)+";"+str(self.Height)+";"+str(self.Slant)+";"+str(self.Level)+";"+str(rot)+";"+str(self.Offset)
            self.listDents.currentItem().setText(s)

    def addDent(self):
        num = str(self.listDents.count()+1)
        rot = self.RotationAngles[self.valueRotation.currentIndex()]
        s = "Dent "+num+" :"+str(self.Length)+";"+str(self.Width)+";"+str(self.Height)+";"+str(self.Slant)+";"+str(self.Level)+";"+str(rot)+";"+str(self.Offset)
        self.listDents.addItem(s)
        self.listDents.setCurrentRow(self.listDents.count()-1)
        self.editDent()

    def removeDent(self):
        if self.listDents.currentItem():
            self.listDents.takeItem(self.listDents.currentRow())

    def editDent(self,item=None):
        if self.listDents.currentItem():
            s = self.listDents.currentItem().text()
            s = s.split(":")[1]
            s = s.split(";")
            self.valueLength.setText(FreeCAD.Units.Quantity(float(s[0]),FreeCAD.Units.Length).UserString)
            self.valueWidth.setText(FreeCAD.Units.Quantity(float(s[1]),FreeCAD.Units.Length).UserString)
            self.valueHeight.setText(FreeCAD.Units.Quantity(float(s[2]),FreeCAD.Units.Length).UserString)
            self.valueSlant.setText(FreeCAD.Units.Quantity(float(s[3]),FreeCAD.Units.Length).UserString)
            self.valueLevel.setText(FreeCAD.Units.Quantity(float(s[4]),FreeCAD.Units.Length).UserString)
            self.valueRotation.setCurrentIndex(self.RotationAngles.index(int(s[5])))
            self.valueOffset.setText(FreeCAD.Units.Quantity(float(s[6]),FreeCAD.Units.Length).UserString)

    def retranslateUi(self, dialog):
        from PySide import QtGui
        self.form.setWindowTitle(translate("Arch", "Precast options"))
        self.labelDents.setText(translate("Arch", "Dents list"))
        self.buttonAdd.setText(translate("Arch", "Add dent"))
        self.buttonRemove.setText(translate("Arch", "Remove dent"))
        self.labelLength.setText(translate("Arch", "Length"))
        self.labelWidth.setText(translate("Arch", "Width"))
        self.labelHeight.setText(translate("Arch", "Height"))
        self.labelSlant.setText(translate("Arch", "Slant"))
        self.labelLevel.setText(translate("Arch", "Level"))
        self.labelRotation.setText(translate("Arch", "Rotation"))
        self.labelOffset.setText(translate("Arch", "Offset"))

    def getValues(self):
        l = []
        for i in range(self.listDents.count()):
            s = self.listDents.item(i).text()
            l.append(s.split(":")[1])
        return l


def makePrecast(precasttype=None,length=0,width=0,height=0,slabtype="",chamfer=0,dentlength=0,dentwidth=0,dentheight=0,dents=[],base=0,holenumber=0,holemajor=0,holeminor=0,holespacing=0,groovenumber=0,groovedepth=0,grooveheight=0,groovespacing=0,risernumber=0,downlength=0,riser=0,tread=0):

    "Creates one of the precast objects in the current document"

    if precasttype == "Beam":
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Beam")
        _PrecastBeam(obj)
        obj.Length = length
        obj.Width = width
        obj.Height = height
        obj.Chamfer = chamfer
        obj.Dents = dents
        obj.DentLength = dentlength
        obj.DentHeight = dentheight
    elif precasttype == "Pillar":
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Pillar")
        _PrecastPillar(obj)
        obj.Length = length
        obj.Width = width
        obj.Height = height
        obj.Chamfer = chamfer
        obj.Dents = dents
        obj.GrooveNumber = groovenumber
        obj.GrooveDepth = groovedepth
        obj.GrooveHeight = grooveheight
        obj.GrooveSpacing = groovespacing
    elif precasttype == "Panel":
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Panel")
        _PrecastPanel(obj)
        obj.Length = length
        obj.Width = width
        obj.Height = height
        obj.Chamfer = chamfer
        obj.DentWidth = dentwidth
        obj.DentHeight = dentheight
    elif precasttype == "Slab":
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Slab")
        _PrecastSlab(obj)
        obj.SlabType = slabtype
        obj.Length = length
        obj.Width = width
        obj.Height = height
        obj.SlabBase = base
        obj.HoleNumber = holenumber
        obj.HoleMajor = holemajor
        obj.HoleMinor = holeminor
        obj.HoleSpacing = holespacing
    elif precasttype == "I-Beam":
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Beam")
        _PrecastIbeam(obj)
        obj.Length = length
        obj.Width = width
        obj.Height = height
        obj.Chamfer = chamfer
        obj.BeamBase = base
    elif precasttype == "Stairs":
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Stairs")
        _PrecastStairs(obj)
        obj.Length = length
        obj.Width = width
        obj.Height = height
        obj.RiserNumber = risernumber
        obj.DownLength = downlength
        obj.Riser = riser
        obj.Tread = tread
    else:
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Precast")
        _Precast(obj)
    if FreeCAD.GuiUp:
        _ViewProviderPrecast(obj.ViewObject)
    return obj



#! python
# -*- coding: utf-8 -*-
# (c) 2008 Werner Mayer LGPL


import Part, math
import FreeCAD, FreeCADGui
App=FreeCAD
Gui=FreeCADGui
from FreeCAD import Base

def makeBottle(myWidth=50.0, myHeight=70.0, myThickness=30.0):
	aPnt1=Base.Vector(-myWidth/2.,0,0)
	aPnt2=Base.Vector(-myWidth/2.,-myThickness/4.,0)
	aPnt3=Base.Vector(0,-myThickness/2.,0)
	aPnt4=Base.Vector(myWidth/2.,-myThickness/4.,0)
	aPnt5=Base.Vector(myWidth/2.,0,0)

	aArcOfCircle = Part.Arc(aPnt2,aPnt3,aPnt4)
	aSegment1=Part.LineSegment(aPnt1,aPnt2)
	aSegment2=Part.LineSegment(aPnt4,aPnt5)

	aEdge1=aSegment1.toShape()
	aEdge2=aArcOfCircle.toShape()
	aEdge3=aSegment2.toShape()
	aWire=Part.Wire([aEdge1,aEdge2,aEdge3])

	aTrsf=Base.Matrix()
	aTrsf.rotateZ(math.pi) # rotate around the z-axis

	aMirroredWire=aWire.copy()
	aMirroredWire.transformShape(aTrsf)
	myWireProfile=Part.Wire([aWire,aMirroredWire])

	myFaceProfile=Part.Face(myWireProfile)
	aPrismVec=Base.Vector(0,0,myHeight)
	myBody=myFaceProfile.extrude(aPrismVec)

	myBody=myBody.makeFillet(myThickness/12.0,myBody.Edges)

	neckLocation=Base.Vector(0,0,myHeight)
	neckNormal=Base.Vector(0,0,1)

	myNeckRadius = myThickness / 4.
	myNeckHeight = myHeight / 10
	myNeck = Part.makeCylinder(myNeckRadius,myNeckHeight,neckLocation,neckNormal)
	myBody = myBody.fuse(myNeck)

	faceToRemove = 0
	zMax = -1.0

	for xp in myBody.Faces:
		surf = xp.Surface
		if type(surf) == Part.Plane:
			z = surf.Position.z
			if z > zMax:
				zMax = z
				faceToRemove = xp

	# This doesn't work for any reason
	myBody = myBody.makeThickness([faceToRemove],-myThickness/50 , 1.e-3)
	myThreading = Part.makeThread(myNeckHeight/10, myNeckRadius*0.06, myHeight/10, myNeckRadius*0.99)
	myThreading.translate(Base.Vector(0,0,myHeight))
	myCompound = Part.Compound([myBody, myThreading])

	return myCompound

def makeBoreHole():
	# create a document if needed
	if App.ActiveDocument is None:
		App.newDocument("Solid")

	Group = App.ActiveDocument.addObject("App::DocumentObjectGroup","Group")
	Group.Label="Bore hole"

	V1 = Base.Vector(0,10,0)
	V2 = Base.Vector(30,10,0)
	V3 = Base.Vector(30,-10,0)
	V4 = Base.Vector(0,-10,0)
	VC1 = Base.Vector(-10,0,0)
	C1 = Part.Arc(V1,VC1,V4)
	# and the second one
	VC2 = Base.Vector(40,0,0)
	C2 = Part.Arc(V2,VC2,V3)
	L1 = Part.LineSegment(V1,V2)
	# and the second one
	L2 = Part.LineSegment(V4,V3)
	S1 = Part.Shape([C1,C2,L1,L2])

	W=Part.Wire(S1.Edges)
	F=Part.Face(W)
	P=F.extrude(Base.Vector(0,0,5))

	# add objects with the shape
	Wire=Group.newObject("Part::Feature","Wire")
	Wire.Shape=W
	Face=Group.newObject("Part::Feature","Face")
	Face.Shape=F
	Prism=Group.newObject("Part::Feature","Extrude")
	Prism.Shape=P

	c=Part.Circle(Base.Vector(0,0,-1),Base.Vector(0,0,1),2.0)
	w=Part.Wire(c.toShape())
	f=Part.Face(w)
	p=f.extrude(Base.Vector(0,0,7))
	P=P.cut(p)

	# add first borer
	Bore1=Group.newObject("Part::Feature","Borer_1")
	Bore1.Shape=p
	Hole1=Group.newObject("Part::Feature","Borer_Hole1")
	Hole1.Shape=P

	c=Part.Circle(Base.Vector(0,-11,2.5),Base.Vector(0,1,0),1.0)
	w=Part.Wire(c.toShape())
	f=Part.Face(w)
	p=f.extrude(Base.Vector(0,22,0))
	P=P.cut(p)

	# add second borer
	Bore2=Group.newObject("Part::Feature","Borer_2")
	Bore2.Shape=p
	Hole2=Group.newObject("Part::Feature","Borer_Hole2")
	Hole2.Shape=P

	App.ActiveDocument.recompute()

	# hide all objects except of the final one
	Gui.ActiveDocument.getObject(Wire.Name).hide()
	Gui.ActiveDocument.getObject(Face.Name).hide()
	Gui.ActiveDocument.getObject(Prism.Name).hide()
	Gui.ActiveDocument.getObject(Bore1.Name).hide()
	Gui.ActiveDocument.getObject(Hole1.Name).hide()
	Gui.ActiveDocument.getObject(Bore2.Name).hide()
	Gui.ActiveDocument.ActiveView.fitAll()



# ***************************************************************************
# *   Copyright (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>        *
# *   Copyright (c) 2009, 2010 Ken Cline <cline@frii.com>                   *
# *   Copyright (c) 2020 FreeCAD Developers                                 *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""This module provides the object code for Draft Wire.
"""
## @package wire
# \ingroup DRAFT
# \brief This module provides the object code for Draft Wire.

import math

import FreeCAD as App

import DraftGeomUtils
import DraftVecUtils

from PySide.QtCore import QT_TRANSLATE_NOOP

from draftutils.utils import get_param

from draftobjects.base import DraftObject



class Wire(DraftObject):
    """The Wire object"""

    def __init__(self, obj):
        super(Wire, self).__init__(obj, "Wire")

        _tip = "The vertices of the wire"
        obj.addProperty("App::PropertyVectorList","Points",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "If the wire is closed or not"
        obj.addProperty("App::PropertyBool","Closed",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The base object is the wire, it's formed from 2 objects"
        obj.addProperty("App::PropertyLink","Base",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The tool object is the wire, it's formed from 2 objects"
        obj.addProperty("App::PropertyLink","Tool",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The start point of this line"
        obj.addProperty("App::PropertyVectorDistance","Start",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The end point of this line"
        obj.addProperty("App::PropertyVectorDistance","End",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The length of this line"
        obj.addProperty("App::PropertyLength","Length",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "Radius to use to fillet the corners"
        obj.addProperty("App::PropertyLength","FilletRadius",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "Size of the chamfer to give to the corners"
        obj.addProperty("App::PropertyLength","ChamferSize",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "Create a face if this object is closed"
        obj.addProperty("App::PropertyBool","MakeFace",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The number of subdivisions of each edge"
        obj.addProperty("App::PropertyInteger","Subdivisions",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))

        _tip = "The area of this object"
        obj.addProperty("App::PropertyArea","Area",
                        "Draft",QT_TRANSLATE_NOOP("App::Property", _tip))
        
        obj.MakeFace = get_param("fillmode",True)
        obj.Closed = False

    def execute(self, obj):
        import Part
        plm = obj.Placement
        if obj.Base and (not obj.Tool):
            if obj.Base.isDerivedFrom("Sketcher::SketchObject"):
                shape = obj.Base.Shape.copy()
                if obj.Base.Shape.isClosed():
                    if hasattr(obj,"MakeFace"):
                        if obj.MakeFace:
                            shape = Part.Face(shape)
                    else:
                        shape = Part.Face(shape)
                obj.Shape = shape
        elif obj.Base and obj.Tool:
            if hasattr(obj.Base,'Shape') and hasattr(obj.Tool,'Shape'):
                if (not obj.Base.Shape.isNull()) and (not obj.Tool.Shape.isNull()):
                    sh1 = obj.Base.Shape.copy()
                    sh2 = obj.Tool.Shape.copy()
                    shape = sh1.fuse(sh2)
                    if DraftGeomUtils.isCoplanar(shape.Faces):
                        shape = DraftGeomUtils.concatenate(shape)
                        obj.Shape = shape
                        p = []
                        for v in shape.Vertexes: p.append(v.Point)
                        if obj.Points != p: obj.Points = p
        elif obj.Points:
            if obj.Points[0] == obj.Points[-1]:
                if not obj.Closed: obj.Closed = True
                obj.Points.pop()
            if obj.Closed and (len(obj.Points) > 2):
                pts = obj.Points
                if hasattr(obj,"Subdivisions"):
                    if obj.Subdivisions > 0:
                        npts = []
                        for i in range(len(pts)):
                            p1 = pts[i]
                            npts.append(pts[i])
                            if i == len(pts)-1:
                                p2 = pts[0]
                            else:
                                p2 = pts[i+1]
                            v = p2.sub(p1)
                            v = DraftVecUtils.scaleTo(v,v.Length/(obj.Subdivisions+1))
                            for j in range(obj.Subdivisions):
                                npts.append(p1.add(App.Vector(v).multiply(j+1)))
                        pts = npts
                shape = Part.makePolygon(pts+[pts[0]])
                if "ChamferSize" in obj.PropertiesList:
                    if obj.ChamferSize.Value != 0:
                        w = DraftGeomUtils.filletWire(shape,obj.ChamferSize.Value,chamfer=True)
                        if w:
                            shape = w
                if "FilletRadius" in obj.PropertiesList:
                    if obj.FilletRadius.Value != 0:
                        w = DraftGeomUtils.filletWire(shape,obj.FilletRadius.Value)
                        if w:
                            shape = w
                try:
                    if hasattr(obj,"MakeFace"):
                        if obj.MakeFace:
                            shape = Part.Face(shape)
                    else:
                        shape = Part.Face(shape)
                except Part.OCCError:
                    pass
            else:
                edges = []
                pts = obj.Points[1:]
                lp = obj.Points[0]
                for p in pts:
                    if not DraftVecUtils.equals(lp,p):
                        if hasattr(obj,"Subdivisions"):
                            if obj.Subdivisions > 0:
                                npts = []
                                v = p.sub(lp)
                                v = DraftVecUtils.scaleTo(v,v.Length/(obj.Subdivisions+1))
                                edges.append(Part.LineSegment(lp,lp.add(v)).toShape())
                                lv = lp.add(v)
                                for j in range(obj.Subdivisions):
                                    edges.append(Part.LineSegment(lv,lv.add(v)).toShape())
                                    lv = lv.add(v)
                            else:
                                edges.append(Part.LineSegment(lp,p).toShape())
                        else:
                            edges.append(Part.LineSegment(lp,p).toShape())
                        lp = p
                try:
                    shape = Part.Wire(edges)
                except Part.OCCError:
                    print("Error wiring edges")
                    shape = None
                if "ChamferSize" in obj.PropertiesList:
                    if obj.ChamferSize.Value != 0:
                        w = DraftGeomUtils.filletWire(shape,obj.ChamferSize.Value,chamfer=True)
                        if w:
                            shape = w
                if "FilletRadius" in obj.PropertiesList:
                    if obj.FilletRadius.Value != 0:
                        w = DraftGeomUtils.filletWire(shape,obj.FilletRadius.Value)
                        if w:
                            shape = w
            if shape:
                obj.Shape = shape
                if hasattr(obj,"Area") and hasattr(shape,"Area"):
                    obj.Area = shape.Area
                if hasattr(obj,"Length"):
                    obj.Length = shape.Length

        obj.Placement = plm
        obj.positionBySupport()
        self.onChanged(obj,"Placement")

    def onChanged(self, obj, prop):
        if prop == "Start":
            pts = obj.Points
            invpl = App.Placement(obj.Placement).inverse()
            realfpstart = invpl.multVec(obj.Start)
            if pts:
                if pts[0] != realfpstart:
                    pts[0] = realfpstart
                    obj.Points = pts

        elif prop == "End":
            pts = obj.Points
            invpl = App.Placement(obj.Placement).inverse()
            realfpend = invpl.multVec(obj.End)
            if len(pts) > 1:
                if pts[-1] != realfpend:
                    pts[-1] = realfpend
                    obj.Points = pts

        elif prop == "Length":
            if obj.Shape and not obj.Shape.isNull():
                if obj.Length.Value != obj.Shape.Length:
                    if len(obj.Points) == 2:
                        v = obj.Points[-1].sub(obj.Points[0])
                        v = DraftVecUtils.scaleTo(v,obj.Length.Value)
                        obj.Points = [obj.Points[0],obj.Points[0].add(v)]

        elif prop == "Placement":
            pl = App.Placement(obj.Placement)
            if len(obj.Points) >= 2:
                displayfpstart = pl.multVec(obj.Points[0])
                displayfpend = pl.multVec(obj.Points[-1])
                if obj.Start != displayfpstart:
                    obj.Start = displayfpstart
                if obj.End != displayfpend:
                    obj.End = displayfpend


_Wire = Wire


# ***************************************************************************
# *   (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>                  *
# *   (c) 2009, 2010 Ken Cline <cline@frii.com>                             *
# *   (c) 2020 Eliud Cabrera Castillo <e.cabrera-castillo@tum.de>           *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful,            *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with FreeCAD; if not, write to the Free Software        *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides tools for creating clones of objects with the Draft Workbench.

The clone is basically a simple copy of the `Shape` of another object,
whether that is a Draft object or any other 3D object.

The Clone's `Shape` can be scaled in size in any direction.

This implementation was developed before the `App::Link` object was created.
In many cases using `App::Link` makes more sense, as this object is
more memory efficient as it reuses the same internal `Shape`
instead of creating a copy of it.
"""
## @package gui_clone
# \ingroup DRAFT
# \brief Provides tools for creating clones of objects.

from PySide.QtCore import QT_TRANSLATE_NOOP

import FreeCAD as App
import FreeCADGui as Gui
import Draft_rc
import draftguitools.gui_base_original as gui_base_original
import draftguitools.gui_tool_utils as gui_tool_utils
import draftutils.todo as todo
from draftutils.messages import _msg
from draftutils.translate import translate, _tr

# The module is used to prevent complaints from code checkers (flake8)
True if Draft_rc.__name__ else False


class Clone(gui_base_original.Modifier):
    """Gui Command for the Clone tool."""

    def __init__(self):
        super(Clone, self).__init__()
        self.moveAfterCloning = False

    def GetResources(self):
        """Set icon, menu and tooltip."""
        _tip = ("Creates a clone of the selected objects.\n"
                "The resulting clone can be scaled in each "
                "of its three directions.")

        return {'Pixmap': 'Draft_Clone',
                'Accel': "C,L",
                'MenuText': QT_TRANSLATE_NOOP("Draft_Clone", "Clone"),
                'ToolTip': QT_TRANSLATE_NOOP("Draft_Clone", _tip)}

    def Activated(self):
        """Execute when the command is called."""
        super(Clone, self).Activated(name=_tr("Clone"))
        if not Gui.Selection.getSelection():
            if self.ui:
                self.ui.selectUi()
                _msg(translate("draft", "Select an object to clone"))
                self.call = \
                    self.view.addEventCallback("SoEvent",
                                               gui_tool_utils.selectObject)
        else:
            self.proceed()

    def proceed(self):
        """Proceed with the command if one object was selected."""
        if self.call:
            self.view.removeEventCallback("SoEvent", self.call)

        if Gui.Selection.getSelection():
            sels = len(Gui.Selection.getSelection())
            Gui.addModule("Draft")
            App.ActiveDocument.openTransaction(translate("Draft", "Clone"))
            nonRepeatList = []
            n = 0
            for obj in Gui.Selection.getSelection():
                if obj not in nonRepeatList:
                    _cmd = "Draft.clone"
                    _cmd += "("
                    _cmd += "FreeCAD.ActiveDocument."
                    _cmd += 'getObject("' + obj.Name + '")'
                    _cmd += ")"
                    Gui.doCommand("c" + str(n) + " = " + _cmd)
                    nonRepeatList.append(obj)
                    n += 1
            App.ActiveDocument.commitTransaction()
            App.ActiveDocument.recompute()
            Gui.Selection.clearSelection()

            objects = App.ActiveDocument.Objects
            for i in range(sels):
                Gui.Selection.addSelection(objects[-(1 + i)])
        self.finish()

    def finish(self, close=False):
        """Terminate the operation of the tool."""
        super(Clone, self).finish(close=False)
        if self.moveAfterCloning:
            todo.ToDo.delay(Gui.runCommand, "Draft_Move")


Draft_Clone = Clone
Gui.addCommand('Draft_Clone', Clone())
